package sk.xref.jarSelfExtractor;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.filechooser.*;
import javax.swing.plaf.basic.*;
import java.util.*;
import java.io.*;
import java.util.jar.*;
import java.util.zip.*;

class Main extends JFrame {

	String 		jarfile;
	String 		slash;
	String 		osname;
	JTextField 	destinationField;

	class BrowseButton extends JButton implements ActionListener {
		BrowseButton() {
			super("Browse");
			addActionListener(this);
		}
		public void actionPerformed( ActionEvent e) {
			JFileChooser fc = new JFileChooser();
			fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			fc.setApproveButtonText("Select");
			getContentPane().add(fc);
			int i = fc.showOpenDialog(Main.this);
			destinationField.setText(fc.getSelectedFile().getPath());
		}
	}

	class ContinueButton extends JButton implements ActionListener {
		ContinueButton() {
			super("Continue");
			addActionListener(this);
		}
		public boolean shouldExtractFile(String fname) {
			String ignorePrefix = "sk/xref/jarSelfExtractor";
			int ignorePrefixLen = ignorePrefix.length();
			int fnameLen = fname.length();
			if (fnameLen < ignorePrefixLen) return(true);
			String prefix = fname.substring(0, ignorePrefixLen);
			//& System.out.println("Checking " + prefix);
			if (prefix.equals(ignorePrefix)) return(false);
			return(true);
		}

		public void fileSetExecPermission(String fname) {
			if (osname.toLowerCase().indexOf("linux") != -1
				|| osname.toLowerCase().indexOf("unix") != -1) {
				String[] chmodcmd = {"chmod", "a+x", fname};
				try {
					Runtime.getRuntime().exec(chmodcmd);
				} catch (Exception e) {}
			}
		}

		public void jarExtractFile(JarInputStream jar, String fname, 
								   ZipEntry zz) throws Exception {
			byte res[] = new byte[0];
			int bufferSize = 100000;
			int i, reslen = 0;
			byte bb[] = new byte[bufferSize];
			int n = 0;
			FileOutputStream out = new FileOutputStream(fname);
			while (n>=0) {
				n = jar.read(bb , 0, bufferSize);
				if (n>=0) out.write(bb, 0, n);
			}
			out.close();
		}

		public void actionPerformed( ActionEvent e) {
			try {
				(new File(destinationField.getText())).mkdirs();	
				FileInputStream jfi = new FileInputStream(jarfile);
				JarInputStream jar = new JarInputStream(jfi);
				JarEntry zz = jar.getNextJarEntry();
				for(; zz != null; zz = jar.getNextJarEntry()) {
					// do not extract yourself
					String zzname = zz.getName();
					if (shouldExtractFile(zzname)) {
						String ozfile =  destinationField.getText() + slash + zzname;
						String ofile;
						if ((! slash.equals("/")) && slash.length()==1) {
							ofile = ozfile.replace('/', slash.charAt(0));
						} else {
							ofile = ozfile;
						}
						if (zz.isDirectory()) {
							//& System.out.println("Creating directory " + ofile);
							(new File(ofile)).mkdirs();
						} else {
							//& System.out.println("Self extracting file " + ofile);
							jarExtractFile(jar, ofile, zz);
							if (zz.getName().equals("xref")) {
								fileSetExecPermission(ofile);
							}
						}
					}
				}
				jar.close();
				JOptionPane.showMessageDialog(Main.this, "Installation complete");
				System.exit(0);
			} catch (Exception jarException) {
				System.out.println("Problem while extracting files from jar");
				jarException.printStackTrace();
			}
		}
	}

	class CancelButton extends JButton implements ActionListener {
		CancelButton() {
			super("Cancel");
			addActionListener(this);
		}
		public void actionPerformed( ActionEvent e) {
			System.exit(0);
		}
	}

	Main() {
		super("Xref-Java2Html installation");
		String defaultInstalDir;

		jarfile = System.getProperty("java.class.path");

		osname = System.getProperty("os.name");
		String defaultSlash;
		if (osname.indexOf("Windows") != -1) {
			defaultInstalDir = "C:";
			defaultSlash = "\\";
		} else {
			defaultInstalDir = System.getProperty("user.home","");
			defaultSlash = "/";
		}
		slash = System.getProperty("file.separator", defaultSlash);
		defaultInstalDir += slash;

		getContentPane().setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		int y = -1;

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 4; gbc.gridheight = 1;
		gbc.weightx = 1000; gbc.weighty = 10; gbc.fill = GridBagConstraints.HORIZONTAL;
		getContentPane().add(new JPanel(), gbc);

		y++;
		JLabel lab = new JLabel("Install Xref-Java2html into directory:", SwingConstants.CENTER);
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 4; gbc.gridheight = 1;
		gbc.weightx = 1000; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		getContentPane().add(lab, gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 4; gbc.gridheight = 1;
		gbc.weightx = 1000; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		getContentPane().add(new JPanel(), gbc);


		y++;
		destinationField = new JTextField(defaultInstalDir + "xref-java2html");
		Container dest = new Container();
		dest.setLayout(new BoxLayout(dest, BoxLayout.X_AXIS));
		dest.add(destinationField);
		dest.add(new BrowseButton());
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 4; gbc.gridheight = 1;
		gbc.weightx = 1000; gbc.weighty = 1000; gbc.fill = GridBagConstraints.HORIZONTAL;
		getContentPane().add(dest, gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 4; gbc.gridheight = 1;
		gbc.weightx = 1000; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		getContentPane().add(new JPanel(), gbc);


		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1000; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		getContentPane().add(new JPanel(), gbc);
		gbc.gridx = 1; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1000; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		getContentPane().add(new ContinueButton(), gbc);
		gbc.gridx = 2; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1000; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		getContentPane().add(new CancelButton(), gbc);
		gbc.gridx = 3; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1000; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		getContentPane().add(new JPanel(), gbc);


		pack();
		setSize(300,200);
		setLocation(200,200);
		setVisible(true);
		
	}

	public static void main(String args[]) {
		new Main();
	}
}
