package sk.xref.java2html;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.filechooser.*;
import javax.swing.plaf.basic.*;
import java.util.*;
import java.io.*;


class Java2HtmlOptions extends Container {

	Dialog 				owner;
	String				initialOptions;
	OptionsTextArea 	options;
	LinkedList 			graphics = new LinkedList();

	interface OptionGraphics {
		public void refreshGraphics();
	}

   	class OptionPlusTextField extends Container 
		implements ActionListener, FocusListener, OptionGraphics {

		String 						option;
		String 						text;
		int 						mode;
		JCheckBox 					jb;
		JTextField 					tf;

		class BrowseButton extends JButton implements ActionListener {
			BrowseButton() {
				super("Browse");
				addActionListener(this);
			}
			public void actionPerformed( ActionEvent e) {
				PathSelectionDialog settingDialog = new PathSelectionDialog(owner, "");
				settingDialog.resultString = tf.getText();
				settingDialog.setVisible(true);
				tf.setText(settingDialog.resultString);
				stringEditFinish();
			}
		}

		void constructor(String name, String option, String text, int mode, JButton browse) {
			this.option = option;
			this.text = text;
			this.mode = mode;
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			jb = new JCheckBox(name + "  ( " + option + " )");
			add(jb);
			jb.addActionListener(this);
			tf = new JTextField(text);
			add(tf);
			tf.addFocusListener(this);
			if (browse != null) {
				add(browse);
			}
			setSize(500,10);
			graphics.add(this);
		}
		OptionPlusTextField(String name, String option, String text) {
			constructor(name, option, text, OptionsString.MODE_TWO_OPTIONS, null);
		}
		OptionPlusTextField(String name, String option, String text, int mode) {
			constructor(name, option, text, mode, null);
		}
		OptionPlusTextField(String name, String option, String text, int mode, boolean browse) {
			if (browse) {
				constructor(name, option, text, mode, new BrowseButton());
			} else {
				constructor(name, option, text, mode, null);
			}
		}

		private String optString1() {
			if (mode == OptionsString.MODE_ONE_OPTIONS) {
				return("\"" + option + text + "\"");
			} else {
				return(option + " \"" + text + "\"");
			}
		}
		private String optString2() {
			if (mode == OptionsString.MODE_ONE_OPTIONS) {
				return(option + text);
			} else {
				return(option + " " + text);
			}
		}

		public void  actionPerformed( ActionEvent e) {
			if (jb.isSelected()) {
				options.addOption(optString1());
			} else {
				options.removeOption(optString1());
				options.removeOption(optString2());
			}
		}
		void stringEditFinish() {
			String newtext = tf.getText();
			if (jb.isSelected()) {
				options.removeOption(optString1());
				options.removeOption(optString2());
				text = newtext;
				options.addOption(optString1());
			} else {
				text = newtext;
				jb.setSelected(true);
				actionPerformed(null);
			}
		}
		public void  focusGained( FocusEvent e) {
		}
		public void  focusLost( FocusEvent e) {
			stringEditFinish();
		}
		public void refreshGraphics() {
			int pos = options.containsOption(option, mode);
			if (pos != -1) {
				jb.setSelected(true);
				text = options.getOptionAfter(pos, option, mode);
				if (text==null) text = "";
				tf.setText(text);
			} else {
				jb.setSelected(false);
			}
		}
	}

   	class OptionCheckBox extends JCheckBox implements ActionListener, OptionGraphics {
		String option;
		OptionCheckBox(String name, String option) {
			super(name + "  ( " + option + " )");
			this.option = option;
			addActionListener(this);
			graphics.add(this);
		}

		public void  actionPerformed( ActionEvent e) {
			if (isSelected()) {
				options.addOption(option);
			} else {
				options.removeOption(option);
			}
		}
		
		public void refreshGraphics() {
			if (options.containsOption(option) != -1) {
				setSelected(true);
			} else {
				setSelected(false);
			}
		}
  	}


	class OptionsTextArea extends JTextArea implements FocusListener {

		OptionsString	optString;
		private int 	posAfterGetOptionAfter;
		
		OptionsTextArea(String opts) {
			super(opts);
			optString = new OptionsString(opts);
			setLineWrap(true);
			addFocusListener(this);
		}

		private void normalizeOptions() {
			optString.optString = getText();
			optString.normalizeOptions();
			setText(optString.optString);
		}

		public void refreshGraphics() {
			normalizeOptions();
			ListIterator ii = graphics.listIterator();
			while (ii.hasNext()) {
				OptionGraphics aa = (OptionGraphics)ii.next();
				aa.refreshGraphics();
			}
		}

		public String getOptionAfter(int ii, String option, int mode) {
			return(optString.getOptionAfter(ii, option, mode));
		}

		public int containsOption(String opt, int mode) {
			return(optString.containsOption(opt, mode));
		}

		public int containsOption(String opt) {
			return(containsOption(opt, OptionsString.MODE_TWO_OPTIONS));
		}

		public void addOption(String opt) {
			optString.addOption(opt);
			setText(optString.optString);
		}

		public void removeOption(String opt) {
			optString.removeOption(opt);
			setText(optString.optString);
		}

		public void  focusGained( FocusEvent e) {
		}
		public void  focusLost( FocusEvent e) {
			normalizeOptions();
			refreshGraphics();
		}
	}

	public void reset() {
		options.setText(initialOptions);
		options.refreshGraphics();
	}

	Java2HtmlOptions(Dialog owner, String opt) {
		super();

		this.owner = owner;
		initialOptions = opt;

		setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		int y = -1;

		/*
		  y++;
		  gbc.gridx = 0; gbc.gridy = y; 
		  gbc.gridwidth = 1; gbc.gridheight = 1;
		  gbc.weightx = 10; gbc.weighty = 10; gbc.fill = GridBagConstraints.BOTH;
		  add(new JPanel(), gbc);
		*/

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionCheckBox("Create xrefs for global symbols", "-htmlgxlist"), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionCheckBox("Create xrefs for local symbols", "-htmllxlist"), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionCheckBox("Link first letter to xrefs", "-htmldirectx"), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionCheckBox("Generate line numbers", "-htmllinenums"), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionCheckBox("Do not highlight symbols by colors", "-htmlnocolors"), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionCheckBox("Do not underline html links", "-htmlnounderline"), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionCheckBox("Generate .html files instead of .java.html", "-htmlcutsuffix"), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionCheckBox("Split xrefs by first letter of symbols", "-refalphahash"), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionCheckBox("Split xrefs by first two letters of symbols", "-refalpha2hash"), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionCheckBox("Do not cross reference local variables", "-no_local"), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionCheckBox("Generate links to javadoc", "-htmlgenjavadoclinks"), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionCheckBox("Java 1.4 mode", "-java1.4"), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionCheckBox("Report all error messages", "-errors"), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionPlusTextField("Tabulator width", "-htmltab=", "8", 
									OptionsString.MODE_ONE_OPTIONS), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionPlusTextField("Color of links", "-htmllinkcolor=", "navy", 
									OptionsString.MODE_ONE_OPTIONS), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionPlusTextField("Color of line numbers", "-htmllinenumcolor=", "000000", 
									OptionsString.MODE_ONE_OPTIONS), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionPlusTextField("Line numbers label", "-htmllinenumlabel=", "line", 
									OptionsString.MODE_ONE_OPTIONS), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionPlusTextField("Runtime library", "-jdkclasspath", 
									Main.jdkcp,
									OptionsString.MODE_TWO_OPTIONS, 
									true), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionPlusTextField("Paths to javadoc files", "-javadocpath", 
									Main.defaultRootDir, 
									OptionsString.MODE_TWO_OPTIONS, 
									true), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionPlusTextField("Paths to source files", "-sourcepath", 
									Main.defaultRootDir, 
									OptionsString.MODE_TWO_OPTIONS, 
									true), gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new OptionPlusTextField("Paths to class files", "-classpath", 
									Main.defaultRootDir,
									OptionsString.MODE_TWO_OPTIONS,
									true), gbc);

		// --------------------

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		add(new JLabel("Option string:"), gbc);

		y++;
		options = new OptionsTextArea(opt);
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1000; gbc.weighty = 1000; gbc.fill = GridBagConstraints.BOTH;
		add(new JScrollPane(options), gbc);

		options.setFont(Main.getFixedFont());
		options.refreshGraphics();
	}
}


