package sk.xref.java2html;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.filechooser.*;
import javax.swing.plaf.basic.*;
import java.util.*;
import java.io.*;
import java.util.jar.*;
import java.util.zip.*;
import java.net.*;


public class Main extends JFrame {

	FileBrowser	fbrowser;
	StopButton  stopButton;

	public static String defaultTargetDir;
	public static String defaultRootDir;
	public static String slash;
	public static String cpsep;
	public static String installationPath;
	public static String tmpDirectoryPath;
	public static String osname;
	public static String jdkcp;
	public static String homeDir;
	public static String configurationFile;
	public static String xrefTaskFile;

	Vector		sources;
	JList 		source;

	String		targets;
	JTextField 	target;

	Vector		cupaths;
	JList 		cupath;

	JTextArea	output;
	JScrollPane outputScroll;

	XrefExtraProcess runningProcess = null;
	byte xj2htmlGif[] = null;
	byte xrefTask[] = null;

	private static final int INPUT_FILE_BUFFER_SIZE = 1024;

	String java2htmlOptions = " -java2html -stderr -html -nostdop -nocxfile -javafilesonly -mf20 -htmlgxlist -htmllxlist -htmldirectx -refalphahash -htmllinenums -htmlcutsuffix -htmlgenjavadoclinks -htmllinenumlabel=line. ";

	private String getCutPathOption() {
		int i;
		String s = "";
		int cpn = cupaths.size();
		for(i=0; i<cpn; i++) {
			s += " \"-htmlcutpath=" + (String)cupaths.get(i) + "\"";
		}
		return(s);
	}

	String getCommandLineOptions(boolean useSrcOption) {
		int i;
		String s = java2htmlOptions;
		s += getCutPathOption();
		s += " -d \"" + targets + "\"";
		for(i=0; i<sources.size(); i++) {
			if (useSrcOption) {
				s += " \"-src=" + (String)sources.get(i) + "\"";
			} else {
				s += " \"" + (String)sources.get(i) + "\"";
			}
		}
		return(s);
	}

	void processReadedOptions(String ss) {
		int 		pos;
		String 		opt, opts;
		OptionsString oo = new OptionsString(ss);
		oo.normalizeOptions();
		if ((pos=oo.containsOption("-d")) != -1) {
			String tt = oo.getOptionAfter(pos, "-d", OptionsString.MODE_TWO_OPTIONS);
			targets = tt;
			oo.removeOption("-d " + tt);
			oo.removeOption("-d \"" + tt + "\"");
		}
		target.setText(targets);

		cupaths = new Vector();
		while ((pos=oo.containsOption("-htmlcutpath=", OptionsString.MODE_ONE_OPTIONS)) != -1) {
			String tt = oo.getOptionAfter(pos, "-htmlcutpath=", OptionsString.MODE_ONE_OPTIONS);
			cupaths.add(tt);
			oo.removeOption("-htmlcutpath=" + tt);
			oo.removeOption("\"-htmlcutpath=" + tt + "\"");
		}
		cupath.setListData(cupaths);

		
		sources = new Vector();
		while ((pos=oo.containsOption("-src=", OptionsString.MODE_ONE_OPTIONS)) != -1) {
			String tt = oo.getOptionAfter(pos, "-src=", OptionsString.MODE_ONE_OPTIONS);
			sources.add(tt);
			oo.removeOption("-src=" + tt);
			oo.removeOption("\"-src=" + tt + "\"");
		}
		source.setListData(sources);

		java2htmlOptions = oo.optString;

	}

	class AddSource extends JButton implements ActionListener {
		AddSource() {
			super("Add");
			addActionListener(this);
		}
		public void actionPerformed( ActionEvent e) {
			String[] pp = fbrowser.tree.selection(false);
			if (pp!=null) {
				for(int i=0; i<pp.length; i++) {
					if (! sources.contains(pp[i])) {
						sources.add(pp[i]);
					}
				}
				source.setListData(sources);
			}
		}
	}

	class RemoveSource extends JButton implements ActionListener {
		RemoveSource () {
			super("Del");
			addActionListener(this);
		}
		public void actionPerformed( ActionEvent e) {
			Object pp[] = source.getSelectedValues();
			if (pp!=null) {
				for(int i=0; i<pp.length; i++) {
					sources.remove(pp[i]);
				}
				source.setListData(sources);
			}
		}
	}

	class SetTarget extends JButton implements ActionListener, FocusListener {
		SetTarget() {
			super("Set");
			addActionListener(this);
		}
		public void actionPerformed( ActionEvent e) {
			String[] pp = fbrowser.tree.selection(true);
			if (pp!=null && pp.length>0) {
				targets = pp[0];
				target.setText(targets);
			}
		}
		public void  focusGained( FocusEvent e) {
		}
		public void  focusLost( FocusEvent e) {
			targets = target.getText();
		}
	}

	class AddCut extends JButton implements ActionListener {
		AddCut() {
			super("Add");
			addActionListener(this);
		}
		public void actionPerformed( ActionEvent e) {
			String[] pp = fbrowser.tree.selection(true);
			if (pp!=null) {
				for(int i=0; i<pp.length; i++) {
					if (! cupaths.contains(pp[i])) {
						cupaths.add(pp[i]);
					}
				}
				cupath.setListData(cupaths);
			}
		}
	}

	class RemoveCut extends JButton implements ActionListener {
		RemoveCut() {
			super("Del");
			addActionListener(this);
		}
		public void actionPerformed( ActionEvent e) {
			Object pp[] = cupath.getSelectedValues();
			if (pp!=null) {
				for(int i=0; i<pp.length; i++) {
					cupaths.remove(pp[i]);
				}
				cupath.setListData(cupaths);
			}
		}
	}

	static String[] commandTokenize(String ss) {
		OptionsString oo = new OptionsString(ss);
		return(oo.tokenize());
	}

	class RunJava2Html extends JButton implements ActionListener {
		RunJava2Html() {
			super("Run");
			addActionListener(this);
		}

		public void actionPerformed( ActionEvent e) {
			int i;
			String s = getCommandLineOptions(false);
			if (runningProcess!=null) runningProcess.kill();
			String[] command = commandTokenize(  "\"" + installationPath + xrefTaskFile 
											   + "\" " + s);
			runningProcess = new XrefExtraProcessJava2Html(Main.this, command, "xref" + s);
		}
	}

	class StopButton extends JButton implements ActionListener {
		StopButton() {
			super("Stop");
			addActionListener(this);
		}

		public void actionPerformed( ActionEvent e) {
			int i;
			if (runningProcess!=null) runningProcess.kill();
		}
	}

	class OptionsButton extends JButton implements ActionListener {
		OptionsButton() {
			super("Options");
			addActionListener(this);
		}
		public void actionPerformed( ActionEvent e) {
			Options oo = new Options(Main.this);
			oo.setVisible(true);
		}
	}

	public static String getConfigurationFileName() {
		return(configurationFile);
	}

	class Exit extends JButton implements ActionListener {
		Exit() {
			super("Exit");
			addActionListener(this);
		}

		public void actionPerformed( ActionEvent e) {
			int a = JOptionPane.showInternalConfirmDialog(Main.this.getContentPane(), 
														  "Save settings and quit?", 
														  "Quit", JOptionPane.YES_NO_OPTION);
			if (a==0) {
				String optfile = getConfigurationFileName();
				try {
					File ff = new File(optfile);
					File dir = ff.getParentFile();
					if (! dir.isDirectory()) dir.mkdir();
					FileWriter f = new FileWriter(ff);
					f.write(getCommandLineOptions(true));
					f.close();
				} catch (Exception ee) {
					System.err.println("[xref-java2html] Problem while saving configuration.");
				}
				System.exit(0);
			}
		}
	}

	class Close extends WindowAdapter {
		public void windowClosing( WindowEvent e) {
			System.exit(0);
		}
	}

	class Stc extends Container {

		Stc() {
			int height;
			setLayout(new GridBagLayout());
			GridBagConstraints gbc = new GridBagConstraints();
			int y = -1;

			// ---------------------- source
			y++;
			gbc.gridx = 1; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.NONE;
			add(new JPanel(), gbc);
			gbc.gridx = 2; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.NONE;
			add(new JLabel("Sources:"), gbc);

			y++;
			gbc.gridx = 1; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 10; gbc.fill = GridBagConstraints.BOTH;
			add(new JPanel(), gbc);
			y++;
			gbc.gridx = 1; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
			add(new AddSource(), gbc);

			y++;
			gbc.gridx = 1; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
			add(new RemoveSource(), gbc);

			y++;
			gbc.gridx = 1; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 10; gbc.fill = GridBagConstraints.BOTH;
			add(new JPanel(), gbc);

			height = 4;
			gbc.gridx = 2; gbc.gridy = y-height+1; 
			gbc.gridwidth = 1; gbc.gridheight = height;
			gbc.weightx = 100; gbc.weighty = 5000; gbc.fill = GridBagConstraints.BOTH;
			add(new JScrollPane(source), gbc);


			// ---------------------- target
			y++;
			gbc.gridx = 1; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.BOTH;
			add(new JPanel(), gbc);
			gbc.gridx = 2; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.NONE;
			add(new JLabel("Target Directory:"), gbc);

			y++;
			SetTarget st = new SetTarget();
			target.addFocusListener(st);
			gbc.gridx = 1; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
			add(st, gbc);

			gbc.gridx = 2; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
			add(target, gbc);

			// ---------------------- source path

			y++;
			gbc.gridx = 2; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.NONE;
			add(new JLabel("Cut paths:"), gbc);
			gbc.gridx = 1; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.BOTH;
			add(new JPanel(), gbc);

			y++;
			gbc.gridx = 1; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 10; gbc.fill = GridBagConstraints.BOTH;
			add(new JPanel(), gbc);
			y++;
			gbc.gridx = 1; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
			add(new AddCut(), gbc);

			y++;
			gbc.gridx = 1; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
			add(new RemoveCut(), gbc);

			y++;
			gbc.gridx = 1; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 10; gbc.fill = GridBagConstraints.BOTH;
			add(new JPanel(), gbc);

			height = 4;
			gbc.gridx = 2; gbc.gridy = y-height+1; 
			gbc.gridwidth = 1; gbc.gridheight = height;
			gbc.weightx = 100; gbc.weighty = 10; gbc.fill = GridBagConstraints.BOTH;
			add(new JScrollPane(cupath), gbc);
		}
	}

	class Out extends Container {
		Out() {
			int y, height;
			setLayout(new GridBagLayout());
			GridBagConstraints gbc = new GridBagConstraints();
			y=-1;

			// ---------------------- output
			y++;
			gbc.gridx = 0; gbc.gridy = y; 
			gbc.gridwidth = 6; gbc.gridheight = 1;
			gbc.weightx = 1000; gbc.weighty = 1000; gbc.fill = GridBagConstraints.BOTH;
			add(outputScroll, gbc);

			y++;
			gbc.gridx = 0; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
			add(new JPanel(), gbc);
			gbc.gridx = 1; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
			add(new RunJava2Html(), gbc);
			stopButton = new StopButton();
			stopButton.setEnabled(false);			
			gbc.gridx = 2; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
			add(stopButton, gbc);
			gbc.gridx = 3; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
			add(new OptionsButton(), gbc);
			gbc.gridx = 4; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
			add(new Exit(), gbc);
			gbc.gridx = 5; gbc.gridy = y; 
			gbc.gridwidth = 1; gbc.gridheight = 1;
			gbc.weightx = 1; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
			add(new JPanel(), gbc);
		}
	}

	public void setGlobalValues() {
		String deftmp, defslash, defcpsep, defcfile;

		osname = System.getProperty("os.name");
		homeDir = System.getProperty("user.home","");

		if (osname.indexOf("Windows") != -1) {
			defaultRootDir = "C:\\";
			deftmp = "C:\\";
			defslash = "\\";
			defcpsep = ";";
			defcfile = "java2html.rc";
			xrefTaskFile = "xrefw.exe";
		} else {
			defaultRootDir = "/";
			deftmp = "/tmp/";
			defslash = "/";
			defcpsep = ":";
			defcfile = "java2html.rc";
			xrefTaskFile = "xref";
		}

   		tmpDirectoryPath = System.getProperty("java.io.tmpdir", deftmp);
		slash = System.getProperty("file.separator", defslash);
		cpsep = System.getProperty("path.separator", defcpsep);

		defaultTargetDir = homeDir + slash + "src-html";
		installationPath = "";

		jdkcp = System.getProperty("java.home");
		if (jdkcp == null) {
			jdkcp = defaultRootDir;
		} else {
			jdkcp += slash + "lib" + slash + "rt.jar";
			java2htmlOptions += "-jdkclasspath \"" + jdkcp + "\" ";
		}

		String cp = System.getProperty("java.class.path");
		if (cp!=null) {
			//& JOptionPane.showMessageDialog(this, cp);
			int i = cp.toLowerCase().indexOf("java2html.jar");
			if (i == -1) i = cp.toLowerCase().indexOf("java2h~1.jar");
			if (i == -1) {
				// renamed, try anyway as if it was the only file on cp
				i = cp.lastIndexOf('/');
				if (i != -1) i++;
				else {
					i = cp.lastIndexOf('\\');
					if (i != -1) i++;
				}
			}
			if (i != -1) {
				String ss = cp.substring(0, i);
				int j = ss.lastIndexOf(cpsep);
				if (j != -1) ss = ss.substring(j+1);
				installationPath = ss;
			}
		}
		configurationFile = installationPath + System.getProperty("user.name") + slash + defcfile;
	}
	
	public static Font getFixedFont() {
		JTextField tt = new JTextField("xx");
		Font[] ff = GraphicsEnvironment.getLocalGraphicsEnvironment().getAllFonts();
		String[] wanted = {"Lucida Console"};
		for(int i=0; i<wanted.length; i++) {
			for(int j=0; j<ff.length; j++) {
				if (ff[j].getName().equals(wanted[i])) {
					Font res = new Font(wanted[i], 
										tt.getFont().getStyle(), 
										tt.getFont().getSize());
					return(res);
				}
			}
		}
		return(tt.getFont());
	}

	Main() {
		super("Xref - Java2Html 1.4");
		JPanel p;

		setGlobalValues();
		//& System.out.println(System.getProperty("java.class.path"));
		sources = new java.util.Vector();
		source = new JList(sources);

		targets = defaultTargetDir;
		target = new JTextField(targets);

		cupaths = new Vector();
		cupath = new JList();

		output = new JTextArea("",6,60);
		outputScroll = new JScrollPane(output);

		this.addWindowListener(new Close());

		if (true) {
			try {
				FileReader tf = new FileReader(getConfigurationFileName());
				char [] ibuffer  = new char[INPUT_FILE_BUFFER_SIZE];
				String readOpt = ""; 
				boolean loop = true;
				while (loop) {
					int n = tf.read(ibuffer, 0, INPUT_FILE_BUFFER_SIZE);
					if (n > 0) {
						readOpt = readOpt + new String(ibuffer, 0, n);
					} else {
						loop = false;
					}
				}
				tf.close();
				processReadedOptions(readOpt);
			} catch (Exception e) {
			}
		}

		OptionsString ss = new OptionsString(java2htmlOptions);
		ss.normalizeOptions();
		java2htmlOptions = ss.optString;

		//& target.setEditable(false);
		output.setFont(getFixedFont());

		fbrowser = new FileBrowser(Main.defaultRootDir);

		getContentPane().setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		int y = -1;

		y++;
		//& System.out.println("Installation path == " + installationPath);
		URL url = this.getClass().getResource("j2html.gif");
		JLabel l;
		if (url!=null) l = new JLabel(new ImageIcon(url));
		else l = new JLabel("");
		p = new JPanel();
		p.add(l);
		p.setBackground(Color.white);
		p.setForeground(Color.white);
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1000; gbc.weighty = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
		getContentPane().add(p, gbc);

		y++;
		gbc.gridx = 0; gbc.gridy = y; 
		gbc.gridwidth = 1; gbc.gridheight = 1;
		gbc.weightx = 1000; gbc.weighty = 1000; gbc.fill = GridBagConstraints.BOTH;
		getContentPane().add(new JSplitPane(JSplitPane.VERTICAL_SPLIT, 
											new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, 
														   fbrowser,
														   new Stc()),
											new Out()),
							 gbc);
		pack();
		setSize(800,600);
		setLocation(50,20);
		setVisible(true);
		
	}

	public static void main(String args[]) {
		Main main = new Main();
	}

}
