
/*

Copyright (C) Marian Vittek, Xref-Tech, Bratislava, 1997-2002.
All Rights Reserved.

    THIS FILE IS CONSIDERED A TRADE SECRET BY XREF-TECH.
UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.

*/
#ifndef STDINC__H
#define STDINC__H
#ifdef __WIN32__
#include <windows.h>
#include <direct.h>
#else
#include <unistd.h>
#include <dirent.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#include <string.h>
#include <time.h>
#include <setjmp.h>
#include <sys/types.h>
#include <sys/stat.h>
#endif
#include "head2.h"
#include "proto.h"
void noSuchRecordError(char *rec) {
        if (s_opt.debug || s_opt.err) {
                sprintf(tmpBuff," member %s not found\n", rec);
                error(ERR_ST, tmpBuff);
        }
}
void methodAppliedOnNonClass(char *rec) {
        if (s_opt.debug || s_opt.err) {
                sprintf(tmpBuff," %s not applied on a class\n", rec);
                error(ERR_ST, tmpBuff);
        }
}
void methodNameNotRecognized(char *rec) {
        if (s_opt.debug || s_opt.err) {
                sprintf(tmpBuff," %s not recognized as method name\n", rec);
                error(ERR_ST, tmpBuff);
        }
}
unsigned hashFun(char *ss) {
        register unsigned h = 0;
        register char *s = ss;
        register char c;
        for(c= *s; c ; c= *++s) { h+=c; h+=(h<<10); h^=(h>>6);};
        { h+=(h<<3); h^=(h>>11); h+=(h<<15);};
        return(h);
}
void symTabNAInit(struct symTab *t, int size) {
        register int i;
        t->size = size;
        memset(t->tab, 0, sizeof(S_symbol *)*t->size);
}
void symTabInit(struct symTab *t, int size) {
        register int i;
        {t->tab = (S_symbol **) stackMemoryAlloc((size)*sizeof(S_symbol *)); };
        symTabNAInit(t, size);
}
int symTabIsMember(
                                        struct symTab *t,
                                        S_symbol *elem,
                                        int *position,
                                        S_symbol **original
                        ) {
        unsigned posid;
        S_symbol *pp;
        posid = hashFun(elem->name);
        posid = posid % t->size;
        *position = posid;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 55);};
        for (pp=t->tab[posid]; pp!=NULL; pp=pp->next) {
                if ((pp->b.symType==elem->b.symType && strcmp(pp->name,elem->name)==0)) {
                        *original = pp;
                        return(1);
                }
        }
        *original = NULL;
        return(0);
}
int symTabNextMember(
                                        S_symbol *elem,
                                        S_symbol **original
                        ) {
        S_symbol *pp;
        { if (!(*original)) internalCheckFail("*original", "hashlist.tc", 77);};
        for (pp= (*original)->next; pp!=NULL; pp=pp->next) {
                if ((pp->b.symType==elem->b.symType && strcmp(pp->name,elem->name)==0)) {
                        *original = pp;
                        return(1);
                }
        }
        *original = NULL;
        return(0);
}
int symTabMembEqual(
                                        S_symbol *e1,
                                        S_symbol *e2
                        ) {
        { if (!(e1 && e2)) internalCheckFail("e1 && e2", "hashlist.tc", 93);};
        return((e1->b.symType==e2->b.symType && strcmp(e1->name,e2->name)==0));
}
int symTabDelete(
                                        struct symTab *t,
                                        S_symbol *elem
                        ) {
        unsigned posid;
        S_symbol **pp;
        posid = hashFun(elem->name);
        posid = posid % t->size;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 106);};
        for (pp= &(t->tab[posid]); *pp!=NULL; pp= &((*pp)->next)) {
                if (((*pp)->b.symType==elem->b.symType && strcmp((*pp)->name,elem->name)==0)) {
                        *pp = (*pp)->next;
                        return(1);
                }
        }
        return(0);
}
int symTabDeleteExact(
                                        struct symTab *t,
                                        S_symbol *elem
                        ) {
        unsigned posid;
        S_symbol **pp;
        posid = hashFun(elem->name);
        posid = posid % t->size;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 125);};
        for (pp= &(t->tab[posid]); *pp!=NULL; pp= &((*pp)->next)) {
                if ((*pp) == elem) {
                        *pp = (*pp)->next;
                        return(1);
                }
        }
        return(0);
}
int symTabDeleteLast(
                                        struct symTab *t,
                                        S_symbol *elem
                        ) {
        unsigned posid;
        S_symbol **pp, **dd;
        posid = hashFun(elem->name);
        posid = posid % t->size;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 144);};
        dd = NULL;
        for (pp= &(t->tab[posid]); *pp!=NULL; pp= &((*pp)->next)) {
                if (((*pp)->b.symType==elem->b.symType && strcmp((*pp)->name,elem->name)==0)) {
                        dd = pp;
                }
        }
        if (dd!=NULL) {
                *dd = (*dd)->next;
                return(1);
        }
        return(0);
}
void symTabSet (
                                        struct symTab *t,
                                        S_symbol *elem,
                                        int position
                                ) {
        elem->next = t->tab[position];
    t->tab[position] = elem;
}
void symTabSetLast (
                                        struct symTab *t,
                                        S_symbol *elem,
                                        int position
                                ) {
        S_symbol **pp;
        for (pp= &(t->tab[position]); *pp!=NULL; pp= &((*pp)->next)) ;
    *pp = elem;
        elem->next = NULL;
}
void symTabAdd (
                                        struct symTab *t,
                                        S_symbol *elem,
                                        int *position
                                ) {
  S_symbol *origin;
  if (! symTabIsMember(t,elem,position,&origin)) {
        symTabSet (t,elem,*position);
  }
}
void symTabFree(struct symTab *t) {
        { };
}
void symTabMap(
                                        struct symTab *t,
                                        void (*fun)(S_symbol *)
                                ) {
        int i;
        S_symbol *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp);
        }
}
void symTabMap2(
                                        struct symTab *t,
                                        void (*fun)(S_symbol *, void *),
                                        void *a
                                ) {
        int i;
        S_symbol *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp, a);
        }
}
void symTabMap3(
                                        struct symTab *t,
                                        void (*fun)(int)
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i] != NULL) (*fun)(i);
        }
}
void symTabMap31(
                                        struct symTab *t,
                                        void (*fun)(int, void *, void *),
                                        void *arg1, void *arg2
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i] != NULL) (*fun)(i,arg1,arg2);
        }
}
void symTabMap32(
                                        struct symTab *t,
                                        void (*fun)(int, void *, void *, void *),
                                        void *arg1, void *arg2, void *arg3
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i] != NULL) (*fun)(i,arg1,arg2,arg3);
        }
}
void symTabMap4(
                                        struct symTab *t,
                                        void (*fun)(S_symbol *, int),
                                        int a
                                ) {
        int i;
        S_symbol *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp, a);
        }
}
void symTabMap5(
                                        struct symTab *t,
                                        void (*fun)(S_symbol *, void *),
                                        void *arg
                                ) {
        int i;
        S_symbol *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp, arg);
        }
}
void symTabStatistics(
                                        struct symTab *t,
                                        int *usedIndexes,
                                        int *elements,
                                        int *maxdeep
                                ) {
        int i,ui,el,dp,mdp,mdpi;
        S_symbol *pp;
        ui= el = dp = mdp = 0;
        mdpi = -1;
        for(i=0; i<t->size; i++) {
                if (t->tab[i]!=NULL) ui ++;
                dp = el;
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) {
                        el++;
                }
                if (mdp < el-dp) {mdp = el-dp; mdpi = i;}
        }
        *usedIndexes = ui;
        *elements = el;
        *maxdeep = mdp;
}
void javaFqtTabNAInit(struct javaFqtTab *t, int size) {
        register int i;
        t->size = size;
        memset(t->tab, 0, sizeof(S_symbolList *)*t->size);
}
void javaFqtTabInit(struct javaFqtTab *t, int size) {
        register int i;
        {t->tab = (S_symbolList **) stackMemoryAlloc((size)*sizeof(S_symbolList *)); };
        javaFqtTabNAInit(t, size);
}
int javaFqtTabIsMember(
                                        struct javaFqtTab *t,
                                        S_symbolList *elem,
                                        int *position,
                                        S_symbolList **original
                        ) {
        unsigned posid;
        S_symbolList *pp;
        posid = hashFun(elem->d->linkName);
        posid = posid % t->size;
        *position = posid;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 55);};
        for (pp=t->tab[posid]; pp!=NULL; pp=pp->next) {
                if (( pp->d->b.symType==elem->d->b.symType && strcmp(pp->d->linkName,elem->d->linkName)==0 )) {
                        *original = pp;
                        return(1);
                }
        }
        *original = NULL;
        return(0);
}
int javaFqtTabNextMember(
                                        S_symbolList *elem,
                                        S_symbolList **original
                        ) {
        S_symbolList *pp;
        { if (!(*original)) internalCheckFail("*original", "hashlist.tc", 77);};
        for (pp= (*original)->next; pp!=NULL; pp=pp->next) {
                if (( pp->d->b.symType==elem->d->b.symType && strcmp(pp->d->linkName,elem->d->linkName)==0 )) {
                        *original = pp;
                        return(1);
                }
        }
        *original = NULL;
        return(0);
}
int javaFqtTabMembEqual(
                                        S_symbolList *e1,
                                        S_symbolList *e2
                        ) {
        { if (!(e1 && e2)) internalCheckFail("e1 && e2", "hashlist.tc", 93);};
        return(( e1->d->b.symType==e2->d->b.symType && strcmp(e1->d->linkName,e2->d->linkName)==0 ));
}
int javaFqtTabDelete(
                                        struct javaFqtTab *t,
                                        S_symbolList *elem
                        ) {
        unsigned posid;
        S_symbolList **pp;
        posid = hashFun(elem->d->linkName);
        posid = posid % t->size;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 106);};
        for (pp= &(t->tab[posid]); *pp!=NULL; pp= &((*pp)->next)) {
                if (( (*pp)->d->b.symType==elem->d->b.symType && strcmp((*pp)->d->linkName,elem->d->linkName)==0 )) {
                        *pp = (*pp)->next;
                        return(1);
                }
        }
        return(0);
}
int javaFqtTabDeleteExact(
                                        struct javaFqtTab *t,
                                        S_symbolList *elem
                        ) {
        unsigned posid;
        S_symbolList **pp;
        posid = hashFun(elem->d->linkName);
        posid = posid % t->size;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 125);};
        for (pp= &(t->tab[posid]); *pp!=NULL; pp= &((*pp)->next)) {
                if ((*pp) == elem) {
                        *pp = (*pp)->next;
                        return(1);
                }
        }
        return(0);
}
int javaFqtTabDeleteLast(
                                        struct javaFqtTab *t,
                                        S_symbolList *elem
                        ) {
        unsigned posid;
        S_symbolList **pp, **dd;
        posid = hashFun(elem->d->linkName);
        posid = posid % t->size;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 144);};
        dd = NULL;
        for (pp= &(t->tab[posid]); *pp!=NULL; pp= &((*pp)->next)) {
                if (( (*pp)->d->b.symType==elem->d->b.symType && strcmp((*pp)->d->linkName,elem->d->linkName)==0 )) {
                        dd = pp;
                }
        }
        if (dd!=NULL) {
                *dd = (*dd)->next;
                return(1);
        }
        return(0);
}
void javaFqtTabSet (
                                        struct javaFqtTab *t,
                                        S_symbolList *elem,
                                        int position
                                ) {
        elem->next = t->tab[position];
    t->tab[position] = elem;
}
void javaFqtTabSetLast (
                                        struct javaFqtTab *t,
                                        S_symbolList *elem,
                                        int position
                                ) {
        S_symbolList **pp;
        for (pp= &(t->tab[position]); *pp!=NULL; pp= &((*pp)->next)) ;
    *pp = elem;
        elem->next = NULL;
}
void javaFqtTabAdd (
                                        struct javaFqtTab *t,
                                        S_symbolList *elem,
                                        int *position
                                ) {
  S_symbolList *origin;
  if (! javaFqtTabIsMember(t,elem,position,&origin)) {
        javaFqtTabSet (t,elem,*position);
  }
}
void javaFqtTabFree(struct javaFqtTab *t) {
        { };
}
void javaFqtTabMap(
                                        struct javaFqtTab *t,
                                        void (*fun)(S_symbolList *)
                                ) {
        int i;
        S_symbolList *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp);
        }
}
void javaFqtTabMap2(
                                        struct javaFqtTab *t,
                                        void (*fun)(S_symbolList *, void *),
                                        void *a
                                ) {
        int i;
        S_symbolList *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp, a);
        }
}
void javaFqtTabMap3(
                                        struct javaFqtTab *t,
                                        void (*fun)(int)
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i] != NULL) (*fun)(i);
        }
}
void javaFqtTabMap31(
                                        struct javaFqtTab *t,
                                        void (*fun)(int, void *, void *),
                                        void *arg1, void *arg2
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i] != NULL) (*fun)(i,arg1,arg2);
        }
}
void javaFqtTabMap32(
                                        struct javaFqtTab *t,
                                        void (*fun)(int, void *, void *, void *),
                                        void *arg1, void *arg2, void *arg3
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i] != NULL) (*fun)(i,arg1,arg2,arg3);
        }
}
void javaFqtTabMap4(
                                        struct javaFqtTab *t,
                                        void (*fun)(S_symbolList *, int),
                                        int a
                                ) {
        int i;
        S_symbolList *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp, a);
        }
}
void javaFqtTabMap5(
                                        struct javaFqtTab *t,
                                        void (*fun)(S_symbolList *, void *),
                                        void *arg
                                ) {
        int i;
        S_symbolList *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp, arg);
        }
}
void javaFqtTabStatistics(
                                        struct javaFqtTab *t,
                                        int *usedIndexes,
                                        int *elements,
                                        int *maxdeep
                                ) {
        int i,ui,el,dp,mdp,mdpi;
        S_symbolList *pp;
        ui= el = dp = mdp = 0;
        mdpi = -1;
        for(i=0; i<t->size; i++) {
                if (t->tab[i]!=NULL) ui ++;
                dp = el;
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) {
                        el++;
                }
                if (mdp < el-dp) {mdp = el-dp; mdpi = i;}
        }
        *usedIndexes = ui;
        *elements = el;
        *maxdeep = mdp;
}
void jslTypeTabNAInit(struct jslTypeTab *t, int size) {
        register int i;
        t->size = size;
        memset(t->tab, 0, sizeof(S_symbolList *)*t->size);
}
void jslTypeTabInit(struct jslTypeTab *t, int size) {
        register int i;
        {t->tab = (S_symbolList **) stackMemoryAlloc((size)*sizeof(S_symbolList *)); };
        jslTypeTabNAInit(t, size);
}
int jslTypeTabIsMember(
                                        struct jslTypeTab *t,
                                        S_symbolList *elem,
                                        int *position,
                                        S_symbolList **original
                        ) {
        unsigned posid;
        S_symbolList *pp;
        posid = hashFun(elem->d->name);
        posid = posid % t->size;
        *position = posid;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 55);};
        for (pp=t->tab[posid]; pp!=NULL; pp=pp->next) {
                if (( pp->d->b.symType==elem->d->b.symType && strcmp(pp->d->name,elem->d->name)==0 )) {
                        *original = pp;
                        return(1);
                }
        }
        *original = NULL;
        return(0);
}
int jslTypeTabNextMember(
                                        S_symbolList *elem,
                                        S_symbolList **original
                        ) {
        S_symbolList *pp;
        { if (!(*original)) internalCheckFail("*original", "hashlist.tc", 77);};
        for (pp= (*original)->next; pp!=NULL; pp=pp->next) {
                if (( pp->d->b.symType==elem->d->b.symType && strcmp(pp->d->name,elem->d->name)==0 )) {
                        *original = pp;
                        return(1);
                }
        }
        *original = NULL;
        return(0);
}
int jslTypeTabMembEqual(
                                        S_symbolList *e1,
                                        S_symbolList *e2
                        ) {
        { if (!(e1 && e2)) internalCheckFail("e1 && e2", "hashlist.tc", 93);};
        return(( e1->d->b.symType==e2->d->b.symType && strcmp(e1->d->name,e2->d->name)==0 ));
}
int jslTypeTabDelete(
                                        struct jslTypeTab *t,
                                        S_symbolList *elem
                        ) {
        unsigned posid;
        S_symbolList **pp;
        posid = hashFun(elem->d->name);
        posid = posid % t->size;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 106);};
        for (pp= &(t->tab[posid]); *pp!=NULL; pp= &((*pp)->next)) {
                if (( (*pp)->d->b.symType==elem->d->b.symType && strcmp((*pp)->d->name,elem->d->name)==0 )) {
                        *pp = (*pp)->next;
                        return(1);
                }
        }
        return(0);
}
int jslTypeTabDeleteExact(
                                        struct jslTypeTab *t,
                                        S_symbolList *elem
                        ) {
        unsigned posid;
        S_symbolList **pp;
        posid = hashFun(elem->d->name);
        posid = posid % t->size;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 125);};
        for (pp= &(t->tab[posid]); *pp!=NULL; pp= &((*pp)->next)) {
                if ((*pp) == elem) {
                        *pp = (*pp)->next;
                        return(1);
                }
        }
        return(0);
}
int jslTypeTabDeleteLast(
                                        struct jslTypeTab *t,
                                        S_symbolList *elem
                        ) {
        unsigned posid;
        S_symbolList **pp, **dd;
        posid = hashFun(elem->d->name);
        posid = posid % t->size;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 144);};
        dd = NULL;
        for (pp= &(t->tab[posid]); *pp!=NULL; pp= &((*pp)->next)) {
                if (( (*pp)->d->b.symType==elem->d->b.symType && strcmp((*pp)->d->name,elem->d->name)==0 )) {
                        dd = pp;
                }
        }
        if (dd!=NULL) {
                *dd = (*dd)->next;
                return(1);
        }
        return(0);
}
void jslTypeTabSet (
                                        struct jslTypeTab *t,
                                        S_symbolList *elem,
                                        int position
                                ) {
        elem->next = t->tab[position];
    t->tab[position] = elem;
}
void jslTypeTabSetLast (
                                        struct jslTypeTab *t,
                                        S_symbolList *elem,
                                        int position
                                ) {
        S_symbolList **pp;
        for (pp= &(t->tab[position]); *pp!=NULL; pp= &((*pp)->next)) ;
    *pp = elem;
        elem->next = NULL;
}
void jslTypeTabAdd (
                                        struct jslTypeTab *t,
                                        S_symbolList *elem,
                                        int *position
                                ) {
  S_symbolList *origin;
  if (! jslTypeTabIsMember(t,elem,position,&origin)) {
        jslTypeTabSet (t,elem,*position);
  }
}
void jslTypeTabFree(struct jslTypeTab *t) {
        { };
}
void jslTypeTabMap(
                                        struct jslTypeTab *t,
                                        void (*fun)(S_symbolList *)
                                ) {
        int i;
        S_symbolList *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp);
        }
}
void jslTypeTabMap2(
                                        struct jslTypeTab *t,
                                        void (*fun)(S_symbolList *, void *),
                                        void *a
                                ) {
        int i;
        S_symbolList *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp, a);
        }
}
void jslTypeTabMap3(
                                        struct jslTypeTab *t,
                                        void (*fun)(int)
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i] != NULL) (*fun)(i);
        }
}
void jslTypeTabMap31(
                                        struct jslTypeTab *t,
                                        void (*fun)(int, void *, void *),
                                        void *arg1, void *arg2
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i] != NULL) (*fun)(i,arg1,arg2);
        }
}
void jslTypeTabMap32(
                                        struct jslTypeTab *t,
                                        void (*fun)(int, void *, void *, void *),
                                        void *arg1, void *arg2, void *arg3
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i] != NULL) (*fun)(i,arg1,arg2,arg3);
        }
}
void jslTypeTabMap4(
                                        struct jslTypeTab *t,
                                        void (*fun)(S_symbolList *, int),
                                        int a
                                ) {
        int i;
        S_symbolList *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp, a);
        }
}
void jslTypeTabMap5(
                                        struct jslTypeTab *t,
                                        void (*fun)(S_symbolList *, void *),
                                        void *arg
                                ) {
        int i;
        S_symbolList *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp, arg);
        }
}
void jslTypeTabStatistics(
                                        struct jslTypeTab *t,
                                        int *usedIndexes,
                                        int *elements,
                                        int *maxdeep
                                ) {
        int i,ui,el,dp,mdp,mdpi;
        S_symbolList *pp;
        ui= el = dp = mdp = 0;
        mdpi = -1;
        for(i=0; i<t->size; i++) {
                if (t->tab[i]!=NULL) ui ++;
                dp = el;
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) {
                        el++;
                }
                if (mdp < el-dp) {mdp = el-dp; mdpi = i;}
        }
        *usedIndexes = ui;
        *elements = el;
        *maxdeep = mdp;
}
void refTabNAInit(struct refTab *t, int size) {
        register int i;
        t->size = size;
        memset(t->tab, 0, sizeof(S_symbolRefItem *)*t->size);
}
void refTabInit(struct refTab *t, int size) {
        register int i;
        {t->tab = (S_symbolRefItem **) stackMemoryAlloc((size)*sizeof(S_symbolRefItem *)); };
        refTabNAInit(t, size);
}
int refTabIsMember(
                                        struct refTab *t,
                                        S_symbolRefItem *elem,
                                        int *position,
                                        S_symbolRefItem **original
                        ) {
        unsigned posid;
        S_symbolRefItem *pp;
        posid = (hashFun(elem->name) + (unsigned)elem->vFunClass);
        posid = posid % t->size;
        *position = posid;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 55);};
        for (pp=t->tab[posid]; pp!=NULL; pp=pp->next) {
                if (( pp->b.symType==elem->b.symType && pp->b.storage==elem->b.storage && pp->b.category==elem->b.category && pp->vApplClass==elem->vApplClass && strcmp(pp->name,elem->name)==0)) {
                        *original = pp;
                        return(1);
                }
        }
        *original = NULL;
        return(0);
}
int refTabNextMember(
                                        S_symbolRefItem *elem,
                                        S_symbolRefItem **original
                        ) {
        S_symbolRefItem *pp;
        { if (!(*original)) internalCheckFail("*original", "hashlist.tc", 77);};
        for (pp= (*original)->next; pp!=NULL; pp=pp->next) {
                if (( pp->b.symType==elem->b.symType && pp->b.storage==elem->b.storage && pp->b.category==elem->b.category && pp->vApplClass==elem->vApplClass && strcmp(pp->name,elem->name)==0)) {
                        *original = pp;
                        return(1);
                }
        }
        *original = NULL;
        return(0);
}
int refTabMembEqual(
                                        S_symbolRefItem *e1,
                                        S_symbolRefItem *e2
                        ) {
        { if (!(e1 && e2)) internalCheckFail("e1 && e2", "hashlist.tc", 93);};
        return(( e1->b.symType==e2->b.symType && e1->b.storage==e2->b.storage && e1->b.category==e2->b.category && e1->vApplClass==e2->vApplClass && strcmp(e1->name,e2->name)==0));
}
int refTabDelete(
                                        struct refTab *t,
                                        S_symbolRefItem *elem
                        ) {
        unsigned posid;
        S_symbolRefItem **pp;
        posid = (hashFun(elem->name) + (unsigned)elem->vFunClass);
        posid = posid % t->size;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 106);};
        for (pp= &(t->tab[posid]); *pp!=NULL; pp= &((*pp)->next)) {
                if (( (*pp)->b.symType==elem->b.symType && (*pp)->b.storage==elem->b.storage && (*pp)->b.category==elem->b.category && (*pp)->vApplClass==elem->vApplClass && strcmp((*pp)->name,elem->name)==0)) {
                        *pp = (*pp)->next;
                        return(1);
                }
        }
        return(0);
}
int refTabDeleteExact(
                                        struct refTab *t,
                                        S_symbolRefItem *elem
                        ) {
        unsigned posid;
        S_symbolRefItem **pp;
        posid = (hashFun(elem->name) + (unsigned)elem->vFunClass);
        posid = posid % t->size;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 125);};
        for (pp= &(t->tab[posid]); *pp!=NULL; pp= &((*pp)->next)) {
                if ((*pp) == elem) {
                        *pp = (*pp)->next;
                        return(1);
                }
        }
        return(0);
}
int refTabDeleteLast(
                                        struct refTab *t,
                                        S_symbolRefItem *elem
                        ) {
        unsigned posid;
        S_symbolRefItem **pp, **dd;
        posid = (hashFun(elem->name) + (unsigned)elem->vFunClass);
        posid = posid % t->size;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashlist.tc", 144);};
        dd = NULL;
        for (pp= &(t->tab[posid]); *pp!=NULL; pp= &((*pp)->next)) {
                if (( (*pp)->b.symType==elem->b.symType && (*pp)->b.storage==elem->b.storage && (*pp)->b.category==elem->b.category && (*pp)->vApplClass==elem->vApplClass && strcmp((*pp)->name,elem->name)==0)) {
                        dd = pp;
                }
        }
        if (dd!=NULL) {
                *dd = (*dd)->next;
                return(1);
        }
        return(0);
}
void refTabSet (
                                        struct refTab *t,
                                        S_symbolRefItem *elem,
                                        int position
                                ) {
        elem->next = t->tab[position];
    t->tab[position] = elem;
}
void refTabSetLast (
                                        struct refTab *t,
                                        S_symbolRefItem *elem,
                                        int position
                                ) {
        S_symbolRefItem **pp;
        for (pp= &(t->tab[position]); *pp!=NULL; pp= &((*pp)->next)) ;
    *pp = elem;
        elem->next = NULL;
}
void refTabAdd (
                                        struct refTab *t,
                                        S_symbolRefItem *elem,
                                        int *position
                                ) {
  S_symbolRefItem *origin;
  if (! refTabIsMember(t,elem,position,&origin)) {
        refTabSet (t,elem,*position);
  }
}
void refTabFree(struct refTab *t) {
        { };
}
void refTabMap(
                                        struct refTab *t,
                                        void (*fun)(S_symbolRefItem *)
                                ) {
        int i;
        S_symbolRefItem *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp);
        }
}
void refTabMap2(
                                        struct refTab *t,
                                        void (*fun)(S_symbolRefItem *, void *),
                                        void *a
                                ) {
        int i;
        S_symbolRefItem *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp, a);
        }
}
void refTabMap3(
                                        struct refTab *t,
                                        void (*fun)(int)
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i] != NULL) (*fun)(i);
        }
}
void refTabMap31(
                                        struct refTab *t,
                                        void (*fun)(int, void *, void *),
                                        void *arg1, void *arg2
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i] != NULL) (*fun)(i,arg1,arg2);
        }
}
void refTabMap32(
                                        struct refTab *t,
                                        void (*fun)(int, void *, void *, void *),
                                        void *arg1, void *arg2, void *arg3
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i] != NULL) (*fun)(i,arg1,arg2,arg3);
        }
}
void refTabMap4(
                                        struct refTab *t,
                                        void (*fun)(S_symbolRefItem *, int),
                                        int a
                                ) {
        int i;
        S_symbolRefItem *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp, a);
        }
}
void refTabMap5(
                                        struct refTab *t,
                                        void (*fun)(S_symbolRefItem *, void *),
                                        void *arg
                                ) {
        int i;
        S_symbolRefItem *pp;
        for(i=0; i<t->size; i++) {
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) (*fun)(pp, arg);
        }
}
void refTabStatistics(
                                        struct refTab *t,
                                        int *usedIndexes,
                                        int *elements,
                                        int *maxdeep
                                ) {
        int i,ui,el,dp,mdp,mdpi;
        S_symbolRefItem *pp;
        ui= el = dp = mdp = 0;
        mdpi = -1;
        for(i=0; i<t->size; i++) {
                if (t->tab[i]!=NULL) ui ++;
                dp = el;
                for(pp=t->tab[i]; pp!=NULL; pp=pp->next) {
                        el++;
                }
                if (mdp < el-dp) {mdp = el-dp; mdpi = i;}
        }
        *usedIndexes = ui;
        *elements = el;
        *maxdeep = mdp;
}
void idTabNAInit(struct idTab *t, int size) {
        int i;
        t->numberOfElements = 0;
        t->size = size;
        memset(t->tab, 0, sizeof(S_fileItem*)*t->size);
}
void idTabInit(struct idTab *t, int size) {
        int i;
        {t->tab = (S_fileItem **) stackMemoryAlloc((size)*sizeof(S_fileItem *)); };
        idTabNAInit(t, size);
}
int idTabIsMember(
                                        struct idTab *t,
                                        S_fileItem *elem,
                                        int *position
                        ) {
        unsigned posid;
        posid = hashFun(elem->name);
        posid = posid % t->size;
        { if (!(t->tab!=NULL)) internalCheckFail("t->tab!=NULL", "hashtab.tc", 57);};
        while (t->tab[posid] != NULL) {
                if ((strcmp(t->tab[posid]->name,elem->name)==0)) {
                        *position = posid;
                        return(1);
                }
                posid=(posid+211) % t->size;
        }
        *position = posid;
        return(0);
}
void idTabSet (
                                        struct idTab *t,
                                        S_fileItem *elem,
                                        int position
                                ) {
    if (t->tab[position]==NULL) {
                t->numberOfElements ++;
            if ((t->numberOfElements * 9) / 8 >= t->size) {
          fatalError(ERR_ST, "[hashAdd] file table overflowed", 65);
            }
        }
    t->tab[position] = elem;
}
void idTabAdd (
                                        struct idTab *t,
                                        S_fileItem *elem,
                                        int *position
                                ) {
  if (! idTabIsMember(t,elem,position)) {
        idTabSet (t,elem,*position);
  }
}
void idTabFree(struct idTab *t) {
        { };
}
void idTabMap(
                                        struct idTab *t,
                                        void (*fun)(S_fileItem *)
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i]!=NULL) (*fun)(t->tab[i]);
        }
}
void idTabMap2(
                                        struct idTab *t,
                                        void (*fun)(S_fileItem *, void *),
                                        void *a
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i]!=NULL) (*fun)(t->tab[i], a);
        }
}
void idTabMap4(
                                        struct idTab *t,
                                        void (*fun)(int, void *, void *),
                                        void *a,
                                        void *b
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i]!=NULL) (*fun)(i, a, b);
        }
}
void idTabMap3(
                                        struct idTab *t,
                                        void (*fun)(S_fileItem *, int)
                                ) {
        int i;
        for(i=0; i<t->size; i++) {
                if (t->tab[i]!=NULL) (*fun)(t->tab[i], i);
        }
}
void stackMemoryInit() {
        s_topBlock = (S_topBlock *) memory;
        { (s_topBlock)->firstFreeIndex = sizeof(S_topBlock); (s_topBlock)->tmpMemoryBasei = 0; (s_topBlock)->trail = NULL; (s_topBlock)->previousTopBlock = NULL;};
}
void *stackMemoryAlloc(int size) {
        register int i;
        i = s_topBlock->firstFreeIndex;
        i = ((char *)(((char*)(memory+i))+(STANDARD_ALLIGNEMENT-1-((((unsigned)(memory+i))-1) & (STANDARD_ALLIGNEMENT-1)))))-memory;
        if (i+size < SIZE_workMemory) {
                s_topBlock->firstFreeIndex = i+size;
                return( & memory[i] );
        } else {
        fatalError(ERR_ST,"i+size > SIZE_workMemory,\n\tworking memory overflowed,\n\tread TROUBLES section of README file\n", 65);
                { if (!(0)) internalCheckFail("0", "misc.c", 160);};
                return(NULL);
        }
}
void *stackMemoryRealloc(void *p, int n, int oldn) {
        { if (!(((char *)p) - memory + oldn == s_topBlock->firstFreeIndex)) internalCheckFail("((char *)p) - memory + oldn == s_topBlock->firstFreeIndex", "misc.c", 166);};
        if (s_topBlock->firstFreeIndex + n - oldn >= SIZE_workMemory) {
        fatalError(ERR_ST,"i+size > SIZE_workMemory,\n\tworking memory overflowed,\n\tread TROUBLES section of README file\n", 65);
                { if (!(0)) internalCheckFail("0", "misc.c", 169);};
        }
        s_topBlock->firstFreeIndex += n - oldn;
        return(p);
}
void *stackMemoryPush(void *p, int size) {
        void *m;
        m = stackMemoryAlloc(size);
        memcpy(m,p,size);
        return(m);
}
void stackMemoryPop(void *p, int size) {
        int i;
        i = s_topBlock->firstFreeIndex;
        if (i-size < 0) {
                fprintf(stderr,"i-size < 0\n"); { if (!(0)) internalCheckFail("0", "misc.c", 186);};
        }
        memcpy(p, & memory[i-size], size);
        s_topBlock->firstFreeIndex = i-size;
}
int *stackMemoryPushInt(int x) {
        return((int*)stackMemoryPush(&x, sizeof(int)));
}
char *stackMemoryPushString(char *s) {
        return((char*)stackMemoryPush(s, strlen(s)+1));
}
void stackMemoryBlockStart() {
        S_topBlock *p,top;
        top = *s_topBlock;
        p = ((S_topBlock*) stackMemoryPush(&top,sizeof(S_topBlock)));
        { (s_topBlock)->firstFreeIndex = s_topBlock->firstFreeIndex; (s_topBlock)->tmpMemoryBasei = tmpWorkMemoryi; (s_topBlock)->trail = s_topBlock->trail; (s_topBlock)->previousTopBlock = p;};
}
void stackMemoryBlockFree() {
        int memi;
        { if (!(s_topBlock && s_topBlock->previousTopBlock)) internalCheckFail("s_topBlock && s_topBlock->previousTopBlock", "misc.c", 217);};
        removeFromTrailUntil(s_topBlock->previousTopBlock->trail);
        { if (!(tmpWorkMemoryi >= s_topBlock->tmpMemoryBasei)) internalCheckFail("tmpWorkMemoryi >= s_topBlock->tmpMemoryBasei", "misc.c", 220);};
        tmpWorkMemoryi = s_topBlock->tmpMemoryBasei;
        memi = s_topBlock->firstFreeIndex;
        * s_topBlock = * s_topBlock->previousTopBlock;
        { if (!(s_topBlock != NULL)) internalCheckFail("s_topBlock != NULL", "misc.c", 227);};
}
void stackMemoryDump() {
        int i;
        fprintf(dumpOut,"start stackMemoryDump\n");
        fprintf(dumpOut,"sorry, not yet implemented ");
        for(i=0; i<s_topBlock->firstFreeIndex; i++) {
                if (i%10 == 0) fprintf(dumpOut," ");
                fprintf(dumpOut,".");
        }
        fprintf(dumpOut,"end stackMemoryDump\n");
}
static void trailDump() {
        S_freeTrail *t;
        fprintf(dumpOut,"\nstart trailDump\n");
        for(t=s_topBlock->trail; t!=NULL; t=t->next) fprintf(dumpOut,"%p ",t);
        fprintf(dumpOut,"\nstop trailDump\n");
}
void addToTrail (void (*a)(void*), void *p) {
        S_freeTrail *t;
        if ((s_topBlock->previousTopBlock == NULL) && (((s_language & (2)) != 0)||((s_language & (8)) != 0))) return;
        t = ((S_freeTrail*) stackMemoryAlloc(sizeof(S_freeTrail)));
        t->action = a;
        t->p = (void **) p;
        t->next = s_topBlock->trail;
        s_topBlock->trail = t;
}
void removeFromTrailUntil(S_freeTrail *untilP) {
        S_freeTrail *p;
        for(p=s_topBlock->trail; untilP<p; p=p->next) {
                { if (!(p!=NULL)) internalCheckFail("p!=NULL", "misc.c", 266);};
                (*(p->action))(p->p);
        }
        if (p!=untilP) {
                error(ERR_INTERNAL,"a block structure mismatch?");
        }
        s_topBlock->trail = p;
}
void symDump(S_symbol *s) {
        fprintf(dumpOut,"[symbol] %s\n",s->name);
}
void typeDump(S_typeModifiers *t) {
        fprintf(dumpOut,"dumpStart\n");
        for(; t!=NULL; t=t->next) {
                fprintf(dumpOut," %x\n",t->m);
        }
        fprintf(dumpOut,"dumpStop\n");
}
void symbolRefItemDump(S_symbolRefItem *ss) {
        fprintf(dumpOut,"%s\t%s %s %d %d %d %d %d\n",
                        ss->name,
                        s_fileTab.tab[ss->vApplClass]->name,
                        s_fileTab.tab[ss->vFunClass]->name,
                        ss->b.symType, ss->b.storage, ss->b.scope,
                        ss->b.accessFlags, ss->b.category);
}
int javaTypeStringSPrint(char *buff, char *str, int nameStyle) {
        int i, bj;
        char *pp;
        i = 0;
        for(pp=str; *pp; pp++) {
                if ( s_language == 4 &&
                                *pp == '.' &&
                                *(pp+1) == '<') {
                        while (*pp && *pp!='>') pp++;
                } else if (*pp == '/' || *pp == '$') {
                        if (nameStyle == SHORT_NAME) i=0;
                        else buff[i++] = '.';
                } else {
                        buff[i++] = *pp;
                }
        }
        buff[i] = 0;
        return(i);
}
void typeSPrint(char *buff, int *size, S_typeModifiers *t,
                                char *name, int dclSepChar, int maxDeep, int typedefexp,
                                int longOrShortName) {
        unsigned u;
        S_symbol *dd; S_symbol *ddd;
        char pref[COMPLETION_STRING_SIZE];
        char post[COMPLETION_STRING_SIZE];
        char type[COMPLETION_STRING_SIZE];
        char tmp[COMPLETION_STRING_SIZE];
        char *ttm, *pp;
        int i,j,par,realsize,r,rr,jj,minInfi,typedefexpFlag;
        typedefexpFlag = typedefexp;
        type[0] = 0;
        i = COMPLETION_STRING_SIZE-1;
        pref[i]=0;
        j=0;
        for(;t!=NULL;t=t->next) {
                par = 0;
                for (;t!=NULL && t->m==TypePointer;t=t->next) {
                        { if (t->typedefin != NULL && typedefexpFlag) { { if (!(t->typedefin->name)) internalCheckFail("t->typedefin->name", "misc.c", 353);}; strcpy(type, t->typedefin->name); goto typebreak; } typedefexpFlag = 1;};
                        pref[--i]='*'; par=1;
                }
                { if (!(i>2)) internalCheckFail("i>2", "misc.c", 356);};
                if (t==NULL) goto typebreak;
                { if (t->typedefin != NULL && typedefexpFlag) { { if (!(t->typedefin->name)) internalCheckFail("t->typedefin->name", "misc.c", 358);}; strcpy(type, t->typedefin->name); goto typebreak; } typedefexpFlag = 1;};
                switch (t->m) {
                case TypeArray:
                        if (par) {pref[--i]='('; post[j++]=')'; }
                        if (((s_language & (4)) != 0)) {
                                pref[--i]=' '; pref[--i]=']'; pref[--i]='[';
                        } else {
                                post[j++]='['; post[j++]=']';
                        }
                        break;
                case TypeFunction:
                        if (par) {pref[--i]='('; post[j++]=')'; }
                        sprintf(post+j,"( ");
                        j += strlen(post+j);
                        if (s_language == 4) {
                                jj = COMPLETION_STRING_SIZE - j - 100;
                                javaSignatureSPrint(post+j, &jj, t->u.sig,longOrShortName);
                                j += jj;
                        } else {
                                for(dd=t->u.f.args; dd!=NULL; dd=dd->next) {
                                        if (dd->b.symType == TypeElipsis) ttm = "...";
                                        else if (dd->name == NULL) ttm = "";
                                        else ttm = dd->name;
                                        if (dd->b.symType == TypeDefault && dd->u.type!=NULL) {
                                                jj = COMPLETION_STRING_SIZE - j - 100;
                                                typeSPrint(post+j,&jj,dd->u.type,ttm,' ',maxDeep-1,1,longOrShortName);
                                                j += jj;
                                        } else {
                                                sprintf(post+j,"%s",ttm);
                                                j += strlen(post+j);
                                        }
                                        if (dd->next!=NULL && j<COMPLETION_STRING_SIZE)
                                                sprintf(post+j,", ");
                                                j += strlen(post+j);
                                }
                        }
                        post[j++]=')';
                        break;
                case TypeStruct: case TypeUnion:
                        if (s_language != 4) {
                                if (t->m == TypeStruct) sprintf(type,"struct ");
                                else sprintf(type,"union ");
                                r = strlen(type);
                        } else r=0;
                        if (t->u.t->name!=NULL) {
                                if (s_language == 4) {
                                        r += javaTypeStringSPrint(type+r, t->u.t->linkName,longOrShortName);
                                } else {
                                        sprintf(type+r,"%s ",t->u.t->name);
                                        r += strlen(type+r);
                                }
                        }
                        if (maxDeep>0) {
                                minInfi = r;
                                sprintf(type+r,"{ ");
                                r += strlen(type+r);
                                { if (!(t->u.t->u.s)) internalCheckFail("t->u.t->u.s", "misc.c", 415);};
                                for(ddd=t->u.t->u.s->records; ddd!=NULL; ddd=ddd->next) {
                                        if (ddd->name == NULL) ttm = "";
                                        else ttm = ddd->name;
                                        rr = COMPLETION_STRING_SIZE - r - 100;
                                        { if (!(ddd->u.type)) internalCheckFail("ddd->u.type", "misc.c", 420);};
                                        typeSPrint(type+r, &rr, ddd->u.type, ttm,' ', maxDeep-1,1,longOrShortName);
                                        r += rr;
                                        if (ddd->next!=NULL && r<COMPLETION_STRING_SIZE) {
                                                sprintf(type+r,"; ");
                                                r += strlen(type+r);
                                        }
                                }
                                sprintf(type+r,"}");
                                r += strlen(type+r);
                                if (r > *size - 100) {
                                        r = minInfi;
                                        type[r] = 0;
                                }
                        }
                        break;
                case TypeEnum:
                        if (t->u.t->name==NULL) sprintf(type,"enum ");
                        else sprintf(type,"enum %s",t->u.t->linkName);
                        r = strlen(type);
                        break;
                default:
                        { if (!(t->m >= 0 && t->m < MAX_TYPE)) internalCheckFail("t->m >= 0 && t->m < MAX_TYPE", "misc.c", 454);};
                        { if (!(strlen(typesName[t->m]) < COMPLETION_STRING_SIZE)) internalCheckFail("strlen(typesName[t->m]) < COMPLETION_STRING_SIZE", "misc.c", 455);};
                        strcpy(type, typesName[t->m]);
                        r = strlen(type);
                        break;
                }
                { if (!(i>2 && j<COMPLETION_STRING_SIZE-3)) internalCheckFail("i>2 && j<COMPLETION_STRING_SIZE-3", "misc.c", 460);};
        }
typebreak:
        post[j]=0;
        realsize = strlen(type) + strlen(pref+i) +
                                strlen(name) + strlen(post) +2;
        if (realsize < *size) {
                if (dclSepChar==' ') {
                        sprintf(buff,"%s %s", type, pref+i);
                } else {
                        sprintf(buff,"%s%c %s", type, dclSepChar, pref+i);
                }
                *size = strlen(buff);
                        sprintf(buff+ *size,"%s", name);
                        *size += strlen(buff+ *size);
                sprintf(buff+ *size,"%s", post);
                *size += strlen(buff+ *size);
        } else {
                *size = 0;
                buff[0]=0;
        }
}
void macDefSPrintf(char *tt, int *size, char *name1, char *name2,
                                                                int argn, char **args) {
        int ii,i,brief=0;
        ii = 0;
        sprintf(tt,"# %s%s",name1,name2);
        ii = strlen(tt);
        { if (!(ii< *size)) internalCheckFail("ii< *size", "misc.c", 505);};
        if (argn != -1) {
                sprintf(tt+ii,"(");
                ii += strlen(tt+ii);
                for(i=0;i<argn;i++) {
                        if (args[i]!=NULL && !brief) {
                                if (strcmp(args[i], s_cppVarArgsName)==0) sprintf(tt+ii,"...");
                                else sprintf(tt+ii,"%s",args[i]);
                                ii += strlen(tt+ii);
                        }
                        if (i+1<argn) {
                                sprintf(tt+ii,", ");
                                ii += strlen(tt+ii);
                        }
                        if (ii+100>= *size) {
                                sprintf(tt+ii,"...");
                                ii += strlen(tt+ii);
                                goto pbreak;
                        }
                }
        pbreak:
                sprintf(tt+ii,")");
                ii += strlen(tt+ii);
        }
        *size = ii;
}
char * string3ConcatInStackMem(char *str1, char *str2, char *str3) {
        int l1,l2,l3;
        char *p,*s;
        l1 = strlen(str1);
        l2 = strlen(str2);
        l3 = strlen(str3);
        {s = (char*) stackMemoryAlloc((l1+l2+l3+1)*sizeof(char)); };
        strcpy(s,str1);
        strcpy(s+l1,str2);
        strcpy(s+l1+l2,str3);
        return(s);
}
char *javaCutClassPathFromFileName(char *fname) {
        S_stringList *cp;
        int len;
        char *res,*ss;
        res = fname;
        ss = strchr(fname, ';');
        if (ss!=NULL) {
                res = ss+1;
                goto fini;
        }
        for(cp=s_javaClassPaths; cp!=NULL; cp=cp->next) {
                len = strlen(cp->d);
                if (fnnCmp(cp->d, fname, len) == 0) {
                        res = fname+len;
                        goto fini;
                }
        }
 fini:
        if (*res=='/' || *res=='\\') res++;
        return(res);
}
char *javaCutSourcePathFromFileName(char *fname) {
        S_stringList *cp;
        int len;
        char *res,*ss;
        res = fname;
        ss = strchr(fname, ';');
        if (ss!=NULL) return(ss+1);
        { char *currentPath, *pp, *ecp; int i, ind; char sourcepathes[MAX_SOURCE_PATH_SIZE]; { if (!(s_javaSourcePaths!=NULL)) internalCheckFail("s_javaSourcePaths!=NULL", "misc.c", 576);}; pp = s_javaSourcePaths; strcpy(sourcepathes, pp); currentPath = sourcepathes; ecp = currentPath+strlen(currentPath); while (currentPath<ecp) { for(ind=0; currentPath[ind]!=0 && currentPath[ind]!=CLASS_PATH_SEPARATOR; ind++) ; currentPath[ind] = 0; i = ind; if (i>0 && currentPath[i-1]==SLASH) currentPath[--i] = 0; { len = strlen(currentPath); if (fnnCmp(currentPath, fname, len) == 0) { res = fname+len; goto fini; } }; currentPath += ind; currentPath++; }};
        if (s_javaStat!=NULL && s_javaStat->namedPackageDir != NULL) {
                len = strlen(s_javaStat->namedPackageDir);
                if (fnnCmp(s_javaStat->namedPackageDir, fname, len) == 0) {
                        res = fname+len;
                        goto fini;
                }
        }
 fini:
        if (*res=='/' || *res=='\\') res++;
        return(res);
}
void javaDotifyFileName(char *ss) {
        char *s, *o, *lp;
        lp = NULL;
        for (s=ss; *s; s++) {
                if (*s == '/' || *s == '\\') *s = '.';
        }
}
static void getClassFqtNameFromFileNum(int fnum, char *ttt) {
        char *dd, *ss;
        ss = javaCutClassPathFromFileName(getRealFileNameStatic(s_fileTab.tab[fnum]->name));
        strcpy(ttt, ss);
        dd = lastOccurenceInString(ttt, '.');
        if (dd!=NULL) *dd=0;
}
void javaGetClassNameFromFileNum(int nn, char *tmpOut, int dotify) {
        char *ss,*s,*o,*lp;
        S_stringList *cp;
        int len;
        getClassFqtNameFromFileNum(nn, tmpOut);
        if (dotify==DOTIFY_NAME) javaDotifyFileName(tmpOut);
}
void javaSignatureSPrint(char *buff, int *size, char *sig, int classstyle) {
        char post[COMPLETION_STRING_SIZE];
        int posti;
        char *ssig;
        int j,bj,typ;
        if (sig == NULL) return;
        j = 0;
        { if (!(*sig == '(')) internalCheckFail("*sig == '('", "misc.c", 630);};
        ssig = sig; posti=0; post[0]=0;
        for(ssig++; *ssig && *ssig!=')'; ssig++) {
                { if (!(j+1 < *size)) internalCheckFail("j+1 < *size", "misc.c", 633);};
                if (j+100 > *size) goto fini;
        switchLabel:
                switch (*ssig) {
                case '[':
                  sprintf(post+posti,"[]");
                  posti += strlen(post+posti);
                  for(ssig++; *ssig && isdigit(*ssig); ssig++) ;
                  goto switchLabel;
                case 'L':
                  bj = j;
                  for(ssig++; *ssig && *ssig!=';'; ssig++) {
                        if (*ssig != '/' && *ssig != '$') buff[j++] = *ssig;
                        else if (classstyle==LONG_NAME) buff[j++] = '.';
                        else j=bj;
                  }
                  break;
                default:
                  typ = s_javaCharCodeBaseTypes[*ssig];
                  { if (!(typ > 0 && typ < MAX_TYPE)) internalCheckFail("typ > 0 && typ < MAX_TYPE", "misc.c", 652);};
                  sprintf(buff+j, "%s", typesName[typ]);
                  j += strlen(buff+j);
                }
                sprintf(buff+j, "%s",post);
                j += strlen(buff+j);
                posti = 0; post[0] = 0;
                if (*(ssig+1)!=')') {
                  sprintf(buff+j, ", ");
                  j += strlen(buff+j);
                }
        }
 fini:
        { if (!(j+1 < *size)) internalCheckFail("j+1 < *size", "misc.c", 665);};
        if (j+100 > *size) {
          j = *size - 100 - 3;
          sprintf(buff+j, "...");
          j += strlen(buff+j);
        }
        buff[j] = 0;
        *size = j;
}
void linkNamePrettyPrint(char *ff, char *javaLinkName, int maxlen,
                                                         int argsStyle) {
  int tlen;
  char *tt;
  tt = strchr(javaLinkName, '!');
  if (tt==NULL) tt = javaLinkName;
  else tt ++;
  for(; *tt && *tt!='('; tt++) {
        if (*tt == '/' || *tt=='\\' || *tt=='$') *ff++ = '.';
        else *ff++ = *tt;
        maxlen--;
        if (maxlen <=0) goto fini;
  }
  if (*tt == '(') {
        tlen = maxlen + 100;
        if (tlen <= 100) goto fini;
        *ff ++ = '('; tlen--;
        javaSignatureSPrint(ff, &tlen, tt, argsStyle);
        ff += tlen;
        *ff ++ = ')';
  }
 fini:
  *ff = 0;
}
char *simpleFileNameFromFileNum(int fnum) {
        return(
                simpleFileName(getRealFileNameStatic(s_fileTab.tab[fnum]->name))
                );
}
void printSymbolLinkNameString( FILE *ff, char *linkName) {
        char ttt[MAX_CX_SYMBOL_SIZE];
        linkNamePrettyPrint(ttt, linkName, MAX_CX_SYMBOL_SIZE, SHORT_NAME);
        fprintf(ff,"%s", ttt);
}
void printClassFqtNameFromClassNum(FILE *ff, int fnum) {
        char ttt[MAX_CX_SYMBOL_SIZE];
        getClassFqtNameFromFileNum(fnum, ttt);
        printSymbolLinkNameString(ff, ttt);
}
void printSymbolLinkName( FILE *ff, S_olSymbolsMenu *ss) {
        if (ss->s.b.symType == TypeCppInclude) {
                fprintf(ff,"%s", simpleFileName(getRealFileNameStatic(
                        s_fileTab.tab[ss->s.vApplClass]->name)));
        } else {
                printSymbolLinkNameString(ff, ss->s.name);
        }
}
void fillTrivialSpecialRefItem( S_symbolRefItem *ddd , char *name) {
        { (&ddd->b)->symType = TypeUnknown; (&ddd->b)->storage = StorageAuto; (&ddd->b)->scope = ScopeAuto; (&ddd->b)->accessFlags = 0x000; (&ddd->b)->category = CatLocal; (&ddd->b)->htmlWasLn = 0;};
        { (ddd)->name = name; (ddd)->fileHash = cxFileHashNumber(name); (ddd)->vApplClass = s_noneFileIndex; (ddd)->vFunClass = s_noneFileIndex; (ddd)->b = ddd->b; (ddd)->refs = NULL; (ddd)->next = NULL;};
}
char *strmcpy(char *dest, char *src) {
        register char *p1,*p2;
        for(p1=dest,p2=src; *p2; p1++, p2++) *p1 = *p2;
        *p1 = 0;
        return(p1);
}
char *lastOccurenceInString(char *ss, int ch) {
        register char *s,*res;
        res = NULL;
        for(s=ss; *s; s++) {
                if (*s == ch) res=s;
        }
        return(res);
}
char * getFileSuffix(char *fn) {
        char *cc;
        if (fn == NULL) return("");
        cc = fn + strlen(fn);
        while (*cc != '.' && *cc != SLASH && cc > fn) cc--;
        return(cc);
}
int fnnCmp(char *ss1, char *ss2, int n) {
        register char *s1,*s2;
        register int i;
        int res;
        res = 0;
#ifndef __WIN32__
        if (s_opt.fileNamesCaseSensitive) return(strncmp(ss1,ss2,n));
#endif
        if (n==0) return(0);
        for(s1=ss1,s2=ss2,i=1; *s1 && *s2 && i<n; s1++,s2++,i++) {
#ifdef __WIN32__
                if ( (*s1 == '/' || *s1 == '\\')
                        && (*s2 == '/' || *s2 == '\\')) continue;
#endif
                if (s_opt.fileNamesCaseSensitive) {
                        if (*s1 != *s2) break;
                } else {
                        if (tolower(*s1) != tolower(*s2)) break;
                }
        }
#ifdef __WIN32__
        if ( (*s1 == '/' || *s1 == '\\')
                && (*s2 == '/' || *s2 == '\\')) {
                res = 0;
        } else
#endif
                if (s_opt.fileNamesCaseSensitive) {
                res = *s1 - *s2;
        } else {
                res = tolower(*s1) - tolower(*s2);
        }
        return(res);
}
int fnCmp(char *ss1, char *ss2) {
        register char *s1,*s2;
        int n;
#ifndef __WIN32__
        if (s_opt.fileNamesCaseSensitive) return(strcmp(ss1,ss2));
#endif
        n = strlen(ss1);
        return(fnnCmp(ss1,ss2,n+1));
}
char * getRealFileNameStatic(char *fn) {
        static char ttt[MAX_FILE_NAME_SIZE];
#ifdef __WIN32__
        WIN32_FIND_DATA fdata;
        HANDLE han;
        int si,di,bdi;
        for(si=0,di=0; fn[si]&&fn[si]!=SLASH; si++,di++) ttt[di]=fn[si];
        if (fn[si]) ttt[di++]=fn[si++];
        while (fn[si] && fn[si]!=';') {
                bdi = di;
                while (fn[si] && fn[si]!=SLASH && fn[si]!=';') {
                        ttt[di] = fn[si];
                        si++; di++;
                }
                ttt[di] = 0;
                han = FindFirstFile(ttt, &fdata);
                if (han == INVALID_HANDLE_VALUE) goto bbreak;
                strcpy(ttt+bdi, fdata.cFileName);
                di = bdi + strlen(ttt+bdi);
                FindClose(han);
                { if (!(di < MAX_FILE_NAME_SIZE-1)) internalCheckFail("di < MAX_FILE_NAME_SIZE-1", "misc.c", 832);};
                ttt[di] = fn[si];
                if (fn[si]) { di++; si++; }
        }
 bbreak:
        strcpy(ttt+di, fn+si);
        return(ttt);
#else
        { if (!(strlen(fn) < MAX_FILE_NAME_SIZE-1)) internalCheckFail("strlen(fn) < MAX_FILE_NAME_SIZE-1", "misc.c", 841);};
        strcpy(ttt,fn);
        return(fn);
#endif
}
#ifdef __WIN32__
int mapPatternFiles(
                                        char *pattern ,
                                        void (*fun)(char *file, char *a1,char *a2,S_completions *a3,void *a4,int *a5),
                                        char *a1, char *a2,
                                        S_completions *a3,
                                        void *a4, int *a5) {
  WIN32_FIND_DATA fdata;
  HANDLE han;
  int res;
  res = 0;
  han = FindFirstFile(pattern, &fdata);
  if (han != INVALID_HANDLE_VALUE) {
        do {
          if ( strcmp(fdata.cFileName,".")!=0
                        && strcmp(fdata.cFileName,"..")!=0) {
                (*fun)(fdata.cFileName, a1, a2, a3, a4, a5);
                res = 1;
          }
        } while (FindNextFile(han,&fdata));
        FindClose(han);
  }
  return(res);
}
#endif
int mapDirectoryFiles(
                char *dirname,
                void (*fun)(char *file, char *a1,char *a2,S_completions *a3,void *a4,int *a5),
                char *a1,
                char *a2,
                S_completions *a3,
                void *a4,
                int *a5
        ){
  int res=0;
#ifdef __WIN32__
        WIN32_FIND_DATA fdata;
        HANDLE han;
        char *s,*d;
        char ttt[MAX_FILE_NAME_SIZE];
        for (s=dirname,d=ttt; *s; s++,d++) {
                if (*s=='/') *d=SLASH;
                else *d = *s;
        }
        { if (!(d-ttt < MAX_FILE_NAME_SIZE-3)) internalCheckFail("d-ttt < MAX_FILE_NAME_SIZE-3", "misc.c", 896);};
        sprintf(d,"%c*",SLASH);
        res = mapPatternFiles( ttt, fun, a1, a2, a3, a4, a5);
#else
        struct stat stt;
        DIR *fd;
        struct dirent *dirbuf;
        if ( stat(dirname,&stt) == 0
                        && (stt.st_mode & S_IFMT) == S_IFDIR
                        && (fd = opendir(dirname)) != NULL) {
                while ((dirbuf=readdir(fd)) != NULL) {
                        if (
#ifndef __QNX__
                                        dirbuf->d_ino != 0 &&
#endif
                                        strcmp(dirbuf->d_name, ".") != 0
                                        && strcmp(dirbuf->d_name, "..") != 0) {
                                (*fun)(dirbuf->d_name, a1, a2, a3, a4, a5);
                                res = 1;
                        }
                }
                closedir(fd);
        }
#endif
        return(res);
}
static char *concatFNameInTmpMemory( char *dirname , char *packfile) {
        char *tt, *fname;
        fname = tmpMemory;
        tt = strmcpy(fname, dirname);
        if (*packfile) {
                *tt = SLASH;
                strcpy(tt+1,packfile);
        }
        return(fname);
}
static int pathsStringContainsPath(char *paths, char *path) {
        { char *currentPath, *pp, *ecp; int i, ind; char sourcepathes[MAX_SOURCE_PATH_SIZE]; { if (!(paths!=NULL)) internalCheckFail("paths!=NULL", "misc.c", 937);}; pp = paths; strcpy(sourcepathes, pp); currentPath = sourcepathes; ecp = currentPath+strlen(currentPath); while (currentPath<ecp) { for(ind=0; currentPath[ind]!=0 && currentPath[ind]!=CLASS_PATH_SEPARATOR; ind++) ; currentPath[ind] = 0; i = ind; if (i>0 && currentPath[i-1]==SLASH) currentPath[--i] = 0; { if (fnCmp(currentPath, path)==0) { return(1); } }; currentPath += ind; currentPath++; }};
        return(0);
}
static int classPathContainsPath(char *path) {
        S_stringList *cp;
        for (cp=s_javaClassPaths; cp!=NULL; cp=cp->next) {
                if (fnCmp(cp->d, path)==0) {
                        return(1);
                }
        }
        return(0);
}
int fileNameHasOneOfSiffixes(char *fname, char *suffs) {
        char *suff;
        suff = getFileSuffix(fname);
        if (suff==NULL) return(0);
        if (*suff == '.') suff++;
        return(pathsStringContainsPath(suffs, suff));
}
void javaMapDirectoryFiles1(
                char *packfile,
                void (*fun)(char *file, char *a1,char *a2,S_completions *a3,void *a4,int *a5),
                S_completions *a1,
                void *a2,
                int *a3
        ){
        S_stringList *cp;
        struct stat stt;
        struct dirent *dirbuf;
        char *tt,*fname;
        int i;
        if (packfile == NULL) packfile = "";
        { char *currentPath, *pp, *ecp; int i, ind; char sourcepathes[MAX_SOURCE_PATH_SIZE]; { if (!(s_javaSourcePaths!=NULL)) internalCheckFail("s_javaSourcePaths!=NULL", "misc.c", 986);}; pp = s_javaSourcePaths; strcpy(sourcepathes, pp); currentPath = sourcepathes; ecp = currentPath+strlen(currentPath); while (currentPath<ecp) { for(ind=0; currentPath[ind]!=0 && currentPath[ind]!=CLASS_PATH_SEPARATOR; ind++) ; currentPath[ind] = 0; i = ind; if (i>0 && currentPath[i-1]==SLASH) currentPath[--i] = 0; { fname = concatFNameInTmpMemory( currentPath, packfile); mapDirectoryFiles(fname,fun,currentPath,packfile,a1,a2,a3); }; currentPath += ind; currentPath++; }};
        for (cp=s_javaClassPaths; cp!=NULL; cp=cp->next) {
                if (! pathsStringContainsPath(s_javaSourcePaths, cp->d)) {
                        { if (!(strlen(cp->d)+strlen(packfile)+2 < SIZE_TMP_MEM)) internalCheckFail("strlen(cp->d)+strlen(packfile)+2 < SIZE_TMP_MEM", "misc.c", 994);};
                        fname = concatFNameInTmpMemory( cp->d, packfile);
                        mapDirectoryFiles(fname,fun,cp->d,packfile,a1,a2,a3);
                }
        }
        for(i=0; i<MAX_JAVA_ZIP_ARCHIVES && s_zipArchivTab[i].fn[0]!=0; i++) {
                javaMapZipDirFile(&s_zipArchivTab[i],packfile,a1,a2,a3,fun,
                                s_zipArchivTab[i].fn,packfile);
        }
        if (s_javaStat->namedPackageDir != NULL) {
                if ((! pathsStringContainsPath(s_javaSourcePaths, s_javaStat->namedPackageDir))
                        && (! classPathContainsPath(s_javaStat->namedPackageDir))) {
                        fname = concatFNameInTmpMemory(s_javaStat->namedPackageDir, packfile);
                        mapDirectoryFiles(fname,fun,s_javaStat->namedPackageDir,packfile,a1,a2,a3);
                }
        }
}
void javaMapDirectoryFiles2(
                S_idIdentList *packid,
                void (*fun)(char *file, char *a1,char *a2,S_completions *a3,void *a4,int *a5),
                S_completions *a1,
                void *a2,
                int *a3
        ){
        char *packfile;
        char dname[MAX_FILE_NAME_SIZE];
        packfile=javaCreateComposedName(NULL,packid,NULL,dname,MAX_FILE_NAME_SIZE);
        javaMapDirectoryFiles1(packfile,fun,a1,a2,a3);
}
static void scanClassFile(char *zip, char *file, void *arg) {
        char ttt[MAX_FILE_NAME_SIZE];
        char *tt, *suff;
        S_symbol *memb;
        S_position pos;
        int cpi, fileInd;
        suff = getFileSuffix(file);
        if (fnCmp(suff, ".class")==0) {
                cpi = s_cache.cpi;
                s_cache.activeCache = 1;
                poseCachePoint(0);
                s_cache.activeCache = 0;
                memb = javaGetFieldClass(file, &tt);
                fileInd = javaCreateClassFileItem( memb);
                if (! s_fileTab.tab[fileInd]->b.cxSaved) {
                        sprintf(ttt, "%s%s", zip, file);
                        { if (!(strlen(ttt) < MAX_FILE_NAME_SIZE-1)) internalCheckFail("strlen(ttt) < MAX_FILE_NAME_SIZE-1", "misc.c", 1049);};
                        { if (!(memb->u.s)) internalCheckFail("memb->u.s", "misc.c", 1051);};
                        javaReadClassFile(ttt, memb, DO_NOT_LOAD_SUPER);
                }
                recoverCachePoint(cpi-1,s_cache.cp[cpi-1].lbcc,0);
        }
}
void jarFileParse() {
        int archi,ii,rr;
        archi = zipIndexArchive(s_input_file_name);
        rr = addFileTabItem(s_input_file_name, &ii);
        { if (!(rr==0)) internalCheckFail("rr==0", "misc.c", 1070);};
        testFileModifTime(ii);
        s_fileTab.tab[ii]->b.cxLoading = 1;
        if (archi>=0 && archi<MAX_JAVA_ZIP_ARCHIVES) {
                fsRecMapOnFiles(s_zipArchivTab[archi].dir, s_zipArchivTab[archi].fn,
                                                "", scanClassFile, NULL);
        }
        s_fileTab.tab[ii]->b.cxLoaded = 1;
}
void classFileParse() {
        char ttt[MAX_FILE_NAME_SIZE];
        char *t,*tt;
        { if (!(strlen(s_input_file_name) < MAX_FILE_NAME_SIZE-1)) internalCheckFail("strlen(s_input_file_name) < MAX_FILE_NAME_SIZE-1", "misc.c", 1085);};
        strcpy(ttt, s_input_file_name);
        tt = strchr(ttt, ';');
        if (tt==NULL) {
                ttt[0]=0;
                t = s_input_file_name;
        } else {
                *(tt+1) = 0;
                t = strchr(s_input_file_name, ';');
                { if (!(t!=NULL)) internalCheckFail("t!=NULL", "misc.c", 1094);};
                t ++;
        }
        scanClassFile(ttt, t, NULL);
}
#if defined(__APPLE__)
#include <sys/malloc.h>
#else
#include <malloc.h>
#endif
int cxMemoryOverflowHandler(int n) {
        int delta,ofaktor,faktor,oldsize;
        if (cxMemory.b!=NULL) free(cxMemory.b);
        oldsize = cxMemory.size;
        ofaktor = oldsize / cxMemory.chunkSize;
        faktor = (n-1)/cxMemory.chunkSize + 1;
        if (s_opt.cxMemoryFaktor>=1) faktor *= s_opt.cxMemoryFaktor;
        faktor += ofaktor;
        if (ofaktor*2 > faktor) faktor = ofaktor*2;
        cxMemory.size = faktor * cxMemory.chunkSize;
        cxMemory.b = malloc(cxMemory.size);
        return(cxMemory.b!=NULL);
}
