
/*

Copyright (C) Marian Vittek, Xref-Tech, Bratislava, 1997-2002.
All Rights Reserved.

    THIS FILE IS CONSIDERED A TRADE SECRET BY XREF-TECH.
UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.

*/
#ifndef STDINC__H
#define STDINC__H
#ifdef __WIN32__
#include <windows.h>
#include <direct.h>
#else
#include <unistd.h>
#include <dirent.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#include <string.h>
#include <time.h>
#include <setjmp.h>
#include <sys/types.h>
#include <sys/stat.h>
#endif
#include "head2.h"
#include "proto.h"
#ifdef __WIN32__
#endif
typedef union {
        int integer;
        unsigned unsign;
        S_symbol *symbol;
        S_symbolList *symbolList;
        S_typeModifiers *typeModif;
        S_typeModifiersList *typeModifList;
        S_freeTrail *trail;
        S_idIdent *idIdent;
        S_idIdentList *idlist;
        S_exprTokenType exprType;
        S_intPair intpair;
        S_whileExtractData *whiledata;
} YYSTYPE;
extern YYSTYPE yylval;
extern YYSTYPE cyylval;
extern YYSTYPE javayylval;
extern YYSTYPE javaslyylval;
extern YYSTYPE cccyylval;
extern YYSTYPE yaccyylval;
extern YYSTYPE *uniyylval;
int styyerror(char *s) {
        if (strcmp(s,"syntax error")!=0) {
                sprintf(tmpBuff,"YACC error: %s\n",s);
                error(ERR_INTERNAL,tmpBuff);
        }
        if (s_opt.debug || s_opt.err) {
                sprintf(tmpBuff,"on: %s",yytext);
                error(ERR_ST, tmpBuff);
        }
        return(0);
}
int styyErrorRecovery() {
        if (s_opt.debug || s_opt.err) {
                error(ERR_ST, " recovery");
        }
        return(0);
}
void setToNull(void *p) {
        void **pp;
        pp = (void **)p;
        *pp = NULL;
}
void deleteSymDef(void *p) {
        S_symbol *pp;
        S_javaStat *scp;
        pp = (S_symbol *) p;
        { if (s_opt.debug) {fprintf(dumpOut,"deleting %s %s\n",pp->name,pp->linkName);fflush(dumpOut);}};
        if (symTabDelete(s_javaStat->locals,pp)) return;
        if (symTabDelete(s_symTab,pp)==0) {
                { if (!(s_opt.taskRegime)) internalCheckFail("s_opt.taskRegime", "semact.c", 53);};
                if (s_opt.taskRegime != RegimeEditServer) {
                        error(ERR_INTERNAL,"symbol on deletion not found");
                }
        }
}
void unpackPointers(S_symbol *pp) {
        unsigned i;
        for (i=0; i<pp->b.npointers; i++) {
                appendComposedType(&pp->u.type, TypePointer);
        }
        pp->b.npointers=0;
}
void addSymbol(S_symbol *pp, S_symTab *tab) {
        { if (s_opt.debug) { fprintf(dumpOut,"adding symbol %s: %s %s\n",pp->name,typesName[pp->b.symType],storagesName[pp->b.storage]); fflush(dumpOut); }};
        { if (!(pp->b.npointers==0)) internalCheckFail("pp->b.npointers==0", "semact.c", 75);};
        { int i; S_symbol *memb; symTabIsMember(tab,pp,&i,&memb); if ( ((s_language & (16)) != 0) && pp->b.symType==TypeDefault && pp->u.type->m == TypeFunction) { pp->u.type->u.f.thisFunList = &(tab->tab[i]); } symTabSet(tab,pp,i);};
        addToTrail(deleteSymDef, pp );
}
void recFindPush(S_symbol *str, S_recFindStr *rfs) {
        S_symStructSpecific *ss;
        { if (!(str && (str->b.symType==TypeStruct || str->b.symType==TypeUnion))) internalCheckFail("str && (str->b.symType==TypeStruct || str->b.symType==TypeUnion)", "semact.c", 83);};
        if (rfs->recsClassCounter==0) {
                rfs->recsClassCounter++;
        }
        ss = str->u.s;
        rfs->nextRecord = ss->records;
        rfs->currClass = str;
                rfs->st[rfs->sti] = ss->super;
                { if (!(rfs->sti < MAX_INHERITANCE_DEEP)) internalCheckFail("rfs->sti < MAX_INHERITANCE_DEEP", "semact.c", 93);};
                rfs->sti ++;
}
S_recFindStr * iniFind(S_symbol *s, S_recFindStr *rfs) {
        { if (!(s)) internalCheckFail("s", "semact.c", 99);};
        { if (!(s->b.symType == TypeStruct || s->b.symType == TypeUnion)) internalCheckFail("s->b.symType == TypeStruct || s->b.symType == TypeUnion", "semact.c", 100);};
        { if (!(s->u.s)) internalCheckFail("s->u.s", "semact.c", 101);};
        { if (!(rfs)) internalCheckFail("rfs", "semact.c", 102);};
        { (rfs)->baseClass = s; (rfs)->currClass = NULL; (rfs)->nextRecord = NULL; (rfs)->recsClassCounter = s_recFindCl++; (rfs)->sti = 0;};
        recFindPush(s, rfs);
        return(rfs);
}
int javaOuterClassAccessible(S_symbol *cl) {
        S_javaStat *cs, *lcs;
        S_symStructSpecific *nest;
        int i,in,len;
        if (cl->b.accessFlags & 0x001) {
                return(1);
        }
        { if (!(s_javaStat)) internalCheckFail("s_javaStat", "semact.c", 124);};
        if (cl->u.s->currPackage) {
                return(1);
        }
        return(0);
}
static int javaRecordVisible(S_symbol *appcl, S_symbol *funcl, unsigned accessFlags) {
        if (accessFlags & 0x002) {
                if (appcl!=funcl && strcmp(appcl->linkName, funcl->linkName)!=0) return(0);
        }
        return(1);
}
int javaRecordAccessible(S_symbol *appcl, S_symbol *funcl, S_symbol *rec, unsigned recAccessFlags) {
        S_javaStat *cs, *lcs;
        S_symStructSpecific *nest;
        int i,in,len;
        if (funcl == NULL) return(1);
                { if (!(s_javaStat)) internalCheckFail("s_javaStat", "semact.c", 154);};
        if (recAccessFlags & 0x001) {
                        return(1);
        }
        if (recAccessFlags & 0x004) {
                if (funcl->u.s->currPackage) {
                                return(1);
                }
                for (cs=s_javaStat; cs!=NULL && cs->thisClass!=NULL; cs=cs->next) {
                        if (cs->thisClass == funcl) {
                                        return(1);
                        }
                        if (cctIsMember(&cs->thisClass->u.s->casts, funcl, 1)) {
                                        return(1);
                        }
                }
                        return(0);
        }
        if (recAccessFlags & 0x002) {
                for(lcs=cs=s_javaStat; cs!=NULL && cs->thisClass!=NULL; cs=cs->next) {
                        lcs = cs;
                }
                if (lcs!=NULL && lcs->thisClass!=NULL) {
                        len = strlen(lcs->thisClass->linkName);
                        if (strncmp(lcs->thisClass->linkName, funcl->linkName, len)==0) {
                                        return(1);
                        }
                }
                        return(0);
        }
        if (funcl->u.s->currPackage) {
                        return(1);
        }
                return(0);
}
int javaRecordVisibleAndAccessible(S_symbol *applCl, S_symbol *funCl, S_symbol *r) {
        return(
                javaRecordVisible(applCl, funCl, r->b.accessFlags)
                &&
                javaRecordAccessible(applCl, funCl, r, r->b.accessFlags)
                );
}
int findStrRecordSym( S_recFindStr *ss,
                                                char *recname,
                                                S_symbol **res,
                                                int javaClassif,
                                                int accCheck,
                                                int visibilityCheck
        ) {
        S_symbol *s,*r,*cclass;
        S_symbolList *sss;
        int m;
        for(;;) {
                { if (!(ss)) internalCheckFail("ss", "semact.c", 244);};
                cclass = ss->currClass;
                if (cclass!=NULL&&cclass->u.s->recSearchCounter==ss->recsClassCounter){
                        goto nextClass;
                }
                for(r=ss->nextRecord; r!=NULL; r=r->next) {
                        if (recname==NULL || strcmp(r->name,recname)==0) {
                                if (! ((s_language & (4)) != 0)) {
                                        { *res = r; ss->nextRecord = r->next; return(0);};
                                }
                                if (javaClassif!=CLASS_TO_ANY) {
                                        { if (!(r->b.symType == TypeDefault)) internalCheckFail("r->b.symType == TypeDefault", "semact.c", 260);};
                                        { if (!(r->u.type)) internalCheckFail("r->u.type", "semact.c", 261);};
                                        m = r->u.type->m;
                                        if (m==TypeFunction && javaClassif!=CLASS_TO_METHOD) goto nextRecord;
                                        if (m!=TypeFunction && javaClassif==CLASS_TO_METHOD) goto nextRecord;
                                }
                                { if (!(r)) internalCheckFail("r", "semact.c", 268);};
                                if (visibilityCheck == VISIB_CHECK_YES) {
                                        if (! javaRecordVisible(ss->baseClass, cclass, r->b.accessFlags)) {
                                                { ss->nextRecord = NULL; *res = &s_errorSymbol; return(1);};
                                        }
                                }
                                if (accCheck == ACC_CHECK_YES) {
                                        if (! javaRecordAccessible(ss->baseClass, cclass, r, r->b.accessFlags)) {
                                                if (visibilityCheck == VISIB_CHECK_YES) {
                                                        { ss->nextRecord = NULL; *res = &s_errorSymbol; return(1);};
                                                } else {
                                                        goto nextRecord;
                                                }
                                        }
                                }
                                { *res = r; ss->nextRecord = r->next; return(0);};
                        }
                        nextRecord:;
                }
        nextClass:
                if (cclass!=NULL) {
                        cclass->u.s->recSearchCounter = ss->recsClassCounter;
                }
                while (ss->sti>0 && ss->st[ss->sti-1]==NULL) ss->sti--;
                if (ss->sti==0) {
                        { ss->nextRecord = NULL; *res = &s_errorSymbol; return(1);};
                }
                sss = ss->st[ss->sti-1];
                s = sss->d;
                ss->st[ss->sti-1] = sss->next;
                { if (!(s && (s->b.symType==TypeStruct || s->b.symType==TypeUnion))) internalCheckFail("s && (s->b.symType==TypeStruct || s->b.symType==TypeUnion)", "semact.c", 300);};
                recFindPush(s, ss);
        }
}
int findStrRecord( S_symbol *s,
                                        char *recname,
                                        S_symbol **res,
                                        int javaClassif
                                ) {
        S_recFindStr rfs;
        return(findStrRecordSym(iniFind(s,&rfs),recname,res,javaClassif,
                                                        ACC_CHECK_YES,VISIB_CHECK_YES));
}
S_reference *findStrRecordFromSymbol( S_symbol *sym,
                                                                          S_idIdent *record,
                                                                          S_symbol **res,
                                                                          int javaClassif,
                                                                          S_idIdent *super
        ) {
        S_recFindStr rfs;
        S_reference *ref;
        int rr;
        ref = NULL;
        rr = findStrRecordSym(iniFind(sym,&rfs),record->name,res,
                                                  javaClassif, ACC_CHECK_NO, VISIB_CHECK_NO);
        if (rr == RESULT_OK && rfs.currClass!=NULL &&
                ((*res)->b.storage==StorageField
                 || (*res)->b.storage==StorageMethod
                 || (*res)->b.storage==StorageConstructor)){
                { if (!(rfs.currClass->u.s && rfs.baseClass && rfs.baseClass->u.s)) internalCheckFail("rfs.currClass->u.s && rfs.baseClass && rfs.baseClass->u.s", "semact.c", 337);};
                if ((s_opt.ooChecksBits & 00010)==0
                        || javaRecordVisibleAndAccessible(rfs.baseClass, rfs.currClass, *res)) {
                        ref = addCxReference(*res,&record->p,UsageUsed,
                                                                 rfs.currClass->u.s->classFile,
                                                                 rfs.baseClass->u.s->classFile);
                        if (super!=NULL) addThisCxReferences(s_javaStat->classFileInd,&super->p);
                }
        } else if (rr == RESULT_OK) {
                ref = addCxReference(*res,&record->p,UsageUsed, s_noneFileIndex, s_noneFileIndex);
        } else {
                noSuchRecordError(record->name);
        }
        return(ref);
}
S_reference * findStrRecordFromType( S_typeModifiers *str,
                                                                                S_idIdent *record,
                                                                                S_symbol **res,
                                                                                int javaClassif
                                                                        ) {
        S_reference *ref;
        int rr;
        { if (!(str)) internalCheckFail("str", "semact.c", 362);};
        ref = NULL;
        if (str->m != TypeStruct && str->m != TypeUnion) {
                *res = &s_errorSymbol;
                goto fini;
        }
        ref = findStrRecordFromSymbol( str->u.t, record, res, javaClassif, NULL);
fini:
        return(ref);
}
void labelReference(S_idIdent *id, int usage) {
        char ttt[TMP_STRING_SIZE];
        char *tt;
        { if (!(id)) internalCheckFail("id", "semact.c", 376);};
        if (((s_language & (4)) != 0)) {
                { if (!(s_javaStat&&s_javaStat->thisClass&&s_javaStat->thisClass->u.s)) internalCheckFail("s_javaStat&&s_javaStat->thisClass&&s_javaStat->thisClass->u.s", "semact.c", 378);};
                if (s_cp.function!=NULL) {
                        sprintf(ttt,"%x-%s.%s",s_javaStat->thisClass->u.s->classFile,
                                        s_cp.function->name, id->name);
                } else {
                        sprintf(ttt,"%x-.%s", s_javaStat->thisClass->u.s->classFile,
                                        id->name);
                }
        } else if (s_cp.function!=NULL) {
                tt = strmcpy(ttt, s_cp.function->name);
                *tt = '.';
                tt = strcpy(tt+1,id->name);
        } else {
                strcpy(ttt, id->name);
        }
        { if (!(strlen(ttt)<TMP_STRING_SIZE-1)) internalCheckFail("strlen(ttt)<TMP_STRING_SIZE-1", "semact.c", 393);};
        addTrivialCxReference(ttt, TypeLabel,StorageDefault, &id->p, usage);
}
static void setLocalVariableLinkName(struct symbol *p) {
        char ttt[TMP_STRING_SIZE];
        char nnn[TMP_STRING_SIZE];
        int len,tti;
        if (s_opt.cxrefs == OLO_EXTRACT) {
                sprintf(nnn,"%c%s%c", '!', p->name,
                                '!');
                ttt[0] = ' ';
                sprintf(ttt+1,"%s", s_extractStorageName[p->b.storage]);
                tti = strlen(ttt);
                len = TMP_STRING_SIZE - tti;
                typeSPrint(ttt+tti, &len, p->u.type, nnn, '!', 0,1,SHORT_NAME);
                sprintf(ttt+tti+len,"%c%x-%x-%x-%x", '!',
                        p->pos.file,p->pos.line,p->pos.coll, s_count.localVar++);
        } else {
                if ( p->b.storage==StorageExtern
                                || p->b.storage==StorageTypedef
                                || p->b.storage==StorageConstant ) {
                        sprintf(ttt,"%s", p->name);
                } else {
                                sprintf(ttt,"%x-%x-%x%c%s",p->pos.file,p->pos.line,p->pos.coll,
                                                '!', p->name);
                }
        }
        len = strlen(ttt);
        {p->linkName = (char*) stackMemoryAlloc((len+1)*sizeof(char)); };
        strcpy(p->linkName,ttt);
}
static void setStaticFunctionLinkName( S_symbol *p, int usage ) {
        char ttt[TMP_STRING_SIZE];
        int len,ii;
        char *ss,*basefname;
        S_symbol *memb;
                if (usage==UsageDefined) basefname=s_fileTab.tab[p->pos.file]->name;
                else basefname=s_input_file_name;
                sprintf(ttt,"%s!%s", simpleFileName(basefname), p->name);
                len = strlen(ttt);
                { if (!(len < TMP_STRING_SIZE-2)) internalCheckFail("len < TMP_STRING_SIZE-2", "semact.c", 451);};
                {ss = (char*) stackMemoryAlloc((len+1)*sizeof(char)); };
                strcpy(ss, ttt);
                p->linkName = ss;
}
S_symbol *addNewSymbolDef(S_symbol *p, unsigned storage, S_symTab *tab,
                                                  int usage) {
        S_typeModifiers *tt;
        if (p == &s_errorSymbol || p->b.symType==TypeError) return(p);
        { if (!(p && p->b.symType == TypeDefault && p->u.type)) internalCheckFail("p && p->b.symType == TypeDefault && p->u.type", "semact.c", 464);};
        if (p->u.type->m == TypeFunction && p->b.storage == StorageDefault) {
                p->b.storage = StorageExtern;
        }
        if (p->b.storage == StorageDefault) {
                p->b.storage = storage;
        }
        if (p->b.symType==TypeDefault && p->b.storage==StorageTypedef) {
                {tt = (S_typeModifiers*) stackMemoryAlloc(sizeof(S_typeModifiers)); };
                *tt = *p->u.type;
                p->u.type = tt;
                tt->typedefin = p;
        }
        if (! (s_topBlock->previousTopBlock == NULL)) {
                setLocalVariableLinkName(p);
        } else if (p->b.symType==TypeDefault && p->b.storage==StorageStatic) {
                setStaticFunctionLinkName(p, usage);
        }
        addSymbol(p, tab);
        addCxReference(p, &p->pos, usage,s_noneFileIndex, s_noneFileIndex);
        return(p);
}
S_symbol *addNewCopyOfSymbolDef(S_symbol *def, unsigned storage) {
        S_symbol *p;
        p = ((S_symbol*) stackMemoryAlloc(sizeof(S_symbol)));
        *p = *def;
        addNewSymbolDef(p,storage, s_symTab, UsageDefined);
        return(p);
}
S_symbol *addNewDeclaration(
                                                        S_symbol *btype,
                                                        S_symbol *decl,
                                                        unsigned storage,
                                                        S_symTab *tab
                                                        ) {
        int usage;
        if (decl == &s_errorSymbol || btype == &s_errorSymbol
                || decl->b.symType==TypeError || btype->b.symType==TypeError) {
                return(decl);
        }
        { if (!(decl->b.symType == TypeDefault)) internalCheckFail("decl->b.symType == TypeDefault", "semact.c", 510);};
        completeDeclarator(btype, decl);
        usage = UsageDefined;
        if (decl->u.type->m == TypeFunction) usage = UsageDeclared;
        else if (decl->b.storage == StorageExtern) usage = UsageDeclared;
        addNewSymbolDef(decl, storage, tab, usage);
        return(decl);
}
S_typeModifiers *crSimpleTypeMofifier(unsigned t) {
        S_typeModifiers *p;
        { if (!(t>=0 && t<MAX_TYPE)) internalCheckFail("t>=0 && t<MAX_TYPE", "semact.c", 521);};
        if (s_preCrTypesTab[t] == NULL) {
                p = ((S_typeModifiers*) stackMemoryAlloc(sizeof(S_typeModifiers)));
                { (p)->m = t; { (&((&(p)->u)->f))->args = NULL; (&((&(p)->u)->f))->thisFunList = NULL;}; (p)->typedefin = NULL; (p)->next = NULL;};
        } else {
                p = s_preCrTypesTab[t];
        }
        { if (!(p->m == t)) internalCheckFail("p->m == t", "semact.c", 529);};
        return(p);
}
static S_typeModifiers *mergeBaseType(S_typeModifiers *t1,S_typeModifiers *t2){
        unsigned b,r;
        unsigned modif;
        { if (!(t1->m<MODIFIERS_END && t2->m<MODIFIERS_END)) internalCheckFail("t1->m<MODIFIERS_END && t2->m<MODIFIERS_END", "semact.c", 536);};
        b=t1->m; modif=t2->m;
        if (t1->m <= MODIFIERS_START && t2->m <= MODIFIERS_START) return(t2);
        if (t1->m > MODIFIERS_START) {modif = t1->m; b = t2->m; }
        if (t2->m > MODIFIERS_START) {modif = t2->m; b = t1->m; }
        switch (modif) {
        case TmodLong:
                r = typeLongChange[b];
                break;
        case TmodShort:
                r = typeShortChange[b];
                break;
        case TmodSigned:
                r = typeSignedChange[b];
                break;
        case TmodUnsigned:
                r = typeUnsignedChange[b];
                break;
        case TmodShortSigned:
                r = typeSignedChange[b];
                r = typeShortChange[r];
                break;
        case TmodShortUnsigned:
                r = typeUnsignedChange[b];
                r = typeShortChange[r];
                break;
        case TmodLongSigned:
                r = typeSignedChange[b];
                r = typeLongChange[r];
                break;
        case TmodLongUnsigned:
                r = typeUnsignedChange[b];
                r = typeLongChange[r];
                break;
        default: { if (!(0)) internalCheckFail("0", "semact.c", 572);};
        }
        return(crSimpleTypeMofifier(r));
}
static S_typeModifiers * mergeBaseModTypes(S_typeModifiers *t1, S_typeModifiers *t2) {
        { if (!(t1 && t2)) internalCheckFail("t1 && t2", "semact.c", 578);};
        if (t1->m == TypeDefault) return(t2);
        if (t2->m == TypeDefault) return(t1);
        { if (!(t1->m >=0 && t1->m<MAX_TYPE)) internalCheckFail("t1->m >=0 && t1->m<MAX_TYPE", "semact.c", 581);};
        { if (!(t2->m >=0 && t2->m<MAX_TYPE)) internalCheckFail("t2->m >=0 && t2->m<MAX_TYPE", "semact.c", 582);};
        if (s_preCrTypesTab[t2->m] == NULL) return(t2);
        if (s_preCrTypesTab[t1->m] == NULL) return(t1);
        return(mergeBaseType(t1, t2));
}
S_symbol *typeSpecifier2(S_typeModifiers *t) {
        S_symbol *r;
        if (((s_language & (2)) != 0)) {
                {{ { if (!((1) >= 0)) internalCheckFail("(1) >= 0", "semact.c", 592);}; tmpWorkMemoryi = ((char*)(((char*)(tmpWorkMemory+tmpWorkMemoryi))+(STANDARD_ALLIGNEMENT-1-((((unsigned)(tmpWorkMemory+tmpWorkMemoryi))-1) & (STANDARD_ALLIGNEMENT-1))))) - tmpWorkMemory; if (tmpWorkMemoryi+(1)*sizeof(S_symbol) >= SIZE_tmpWorkMemory) { fatalError(ERR_NO_MEMORY,"tmpWorkMemory", 65); } r = (S_symbol*) (tmpWorkMemory + tmpWorkMemoryi); tmpWorkMemoryi += (1)*sizeof(S_symbol);};};
        } else {
                {r = (S_symbol*) stackMemoryAlloc(sizeof(S_symbol)); };
        }
        { (&r->b)->record = 0; (&r->b)->accessFlags = 0; (&r->b)->javaSourceLoaded = 0; (&r->b)->javaFileLoaded = 0; (&r->b)->symType = TypeDefault; (&r->b)->storage = StorageDefault; (&r->b)->npointers = 0;};
        { (r)->name = NULL; (r)->linkName = NULL; (r)->pos = s_noPos; (r)->b = r->b; (r)->u.type = t; (r)->next = NULL;};
        r->u.type = t;
        return(r);
}
S_symbol *typeSpecifier1(unsigned t) {
        S_symbol *r;
        r = typeSpecifier2(crSimpleTypeMofifier(t));
        return(r);
}
void declTypeSpecifier1(S_symbol *d, unsigned t) {
        S_symbol *r;
        { if (!(d && d->u.type)) internalCheckFail("d && d->u.type", "semact.c", 611);};
        d->u.type = mergeBaseModTypes(d->u.type,crSimpleTypeMofifier(t));
}
void declTypeSpecifier2(S_symbol *d, S_typeModifiers *t) {
        unsigned b,rb;
        { if (!(d && d->u.type)) internalCheckFail("d && d->u.type", "semact.c", 617);};
        d->u.type = mergeBaseModTypes(d->u.type, t);
}
void declTypeSpecifier21(S_typeModifiers *t, S_symbol *d) {
        unsigned b;
        { if (!(d && d->u.type)) internalCheckFail("d && d->u.type", "semact.c", 623);};
        d->u.type = mergeBaseModTypes(t, d->u.type);
}
S_typeModifiers *appendComposedType(S_typeModifiers **d, unsigned t) {
        S_typeModifiers *p;
        p = ((S_typeModifiers*) stackMemoryAlloc(sizeof(S_typeModifiers)));
        { (p)->m = t; { (&((&(p)->u)->f))->args = NULL; (&((&(p)->u)->f))->thisFunList = NULL;}; (p)->typedefin = NULL; (p)->next = NULL;};
        { if (((*d))==NULL) (*d) = p; else { register S_typeModifiers *tempnsl; tempnsl = (*d); while (tempnsl->next!=NULL) tempnsl=tempnsl->next; tempnsl->next = p; } };
        return(p);
}
S_typeModifiers *prependComposedType(S_typeModifiers *d, unsigned t) {
        S_typeModifiers *p;
        p = ((S_typeModifiers*) stackMemoryAlloc(sizeof(S_typeModifiers)));
        { (p)->m = t; { (&((&(p)->u)->f))->args = NULL; (&((&(p)->u)->f))->thisFunList = NULL;}; (p)->typedefin = NULL; (p)->next = d;};
        return(p);
}
void completeDeclarator(S_symbol *t, S_symbol *d) {
        S_typeModifiers *tt,**dt;
        { if (!(t && d)) internalCheckFail("t && d", "semact.c", 656);};
        if (t == &s_errorSymbol || d == &s_errorSymbol
                || t->b.symType==TypeError || d->b.symType==TypeError) return;
        d->b.storage = t->b.storage;
        { if (!(t->b.symType==TypeDefault)) internalCheckFail("t->b.symType==TypeDefault", "semact.c", 660);};
        dt = &(d->u.type); tt = t->u.type;
        if (d->b.npointers) {
                if (d->b.npointers>=1 && (tt->m==TypeStruct||tt->m==TypeUnion)
                        && tt->typedefin==NULL) {
                        d->b.npointers--;
                        { if (!(tt->u.t && tt->u.t->b.symType==tt->m && tt->u.t->u.s)) internalCheckFail("tt->u.t && tt->u.t->b.symType==tt->m && tt->u.t->u.s", "semact.c", 668);};
                        tt = & tt->u.t->u.s->sptrtype;
                } else if (d->b.npointers>=2 && s_preCrPtr2TypesTab[tt->m]!=NULL
                        && tt->typedefin==NULL) {
                        { if (!(tt->next==NULL)) internalCheckFail("tt->next==NULL", "semact.c", 672);};
                        d->b.npointers-=2;
                        tt = s_preCrPtr2TypesTab[tt->m];
                } else if (d->b.npointers>=1 && s_preCrPtr1TypesTab[tt->m]!=NULL
                        && tt->typedefin==NULL) {
                        { if (!(tt->next==NULL)) internalCheckFail("tt->next==NULL", "semact.c", 678);};
                        d->b.npointers--;
                        tt = s_preCrPtr1TypesTab[tt->m];
                }
        }
        unpackPointers(d);
        { if ((*dt)==NULL) *dt = tt; else { register S_typeModifiers *tempnsl; tempnsl = *dt; while (tempnsl->next!=NULL) tempnsl=tempnsl->next; tempnsl->next = tt; } };
}
S_symbol *crSimpleDefinition(unsigned storage, unsigned t, S_idIdent *id) {
        S_typeModifiers *p;
        S_symbol *r;
        p = ((S_typeModifiers*) stackMemoryAlloc(sizeof(S_typeModifiers)));
        { (p)->m = t; { (&((&(p)->u)->f))->args = NULL; (&((&(p)->u)->f))->thisFunList = NULL;}; (p)->typedefin = NULL; (p)->next = NULL;};
        r = ((S_symbol*) stackMemoryAlloc(sizeof(S_symbol)));
        if (id!=NULL) {
                { (&r->b)->record = 0; (&r->b)->accessFlags = 0; (&r->b)->javaSourceLoaded = 0; (&r->b)->javaFileLoaded = 0; (&r->b)->symType = TypeDefault; (&r->b)->storage = storage; (&r->b)->npointers = 0;};
                { (r)->name = id->name; (r)->linkName = id->name; (r)->pos = id->p; (r)->b = r->b; (r)->u.type = p; (r)->next = NULL;};
        } else {
                { (&r->b)->record = 0; (&r->b)->accessFlags = 0; (&r->b)->javaSourceLoaded = 0; (&r->b)->javaFileLoaded = 0; (&r->b)->symType = TypeDefault; (&r->b)->storage = storage; (&r->b)->npointers = 0;};
                { (r)->name = NULL; (r)->linkName = NULL; (r)->pos = s_noPos; (r)->b = r->b; (r)->u.type = p; (r)->next = NULL;};
        }
        r->u.type = p;
        return(r);
}
S_symbolList *crDefinitionList(S_symbol *d) {
        S_symbolList *p;
        { if (!(d)) internalCheckFail("d", "semact.c", 707);};
        p = ((S_symbolList*) stackMemoryAlloc(sizeof(S_symbolList)));
        { (p)->d = d; (p)->next = NULL;};
        return(p);
}
int mergeArguments(S_symbol *id, S_symbol *ty) {
        S_symbol *p;
        int res;
        res = RESULT_OK;
        for(;ty!=NULL; ty=ty->next) {
                if (ty->name != NULL) {
                        for(p=id; p!=NULL; p=p->next) {
                                if (p->name!=NULL && !strcmp(p->name,ty->name)) break;
                        }
                        if (p==NULL) res = RESULT_ERR;
                        else {
                                if (p->u.type == NULL) p->u.type = ty->u.type;
                        }
                }
        }
        return(res);
}
static S_typeModifiers *crSimpleEnumType(S_symbol *edef, int type) {
        S_typeModifiers *res;
        res = ((S_typeModifiers*) stackMemoryAlloc(sizeof(S_typeModifiers)));
        { (res)->m = type; (&(res)->u)->t = edef;; (res)->typedefin = NULL; (res)->next = NULL;};
        res->u.t = edef;
        return(res);
}
S_typeModifiers *simpleStrUnionSpecifier( S_idIdent *typeName,
                                                                                        S_idIdent *id,
                                                                                        int usage
                                                                                ) {
        S_symbol p,*pp;
        int ii,type;
        { if (!(typeName && typeName->sd && typeName->sd->b.symType == TypeKeyword)) internalCheckFail("typeName && typeName->sd && typeName->sd->b.symType == TypeKeyword", "semact.c", 748);};
        { if (!(typeName->sd->u.keyWordVal == 319 || typeName->sd->u.keyWordVal == 328 || typeName->sd->u.keyWordVal == 320)) internalCheckFail("typeName->sd->u.keyWordVal == 319 || typeName->sd->u.keyWordVal == 328 || typeName->sd->u.keyWordVal == 320", "semact.c", 749);};
        if (typeName->sd->u.keyWordVal != 320) type = TypeStruct;
        else type = TypeUnion;
        { (&p.b)->record = 0; (&p.b)->accessFlags = 0; (&p.b)->javaSourceLoaded = 0; (&p.b)->javaFileLoaded = 0; (&p.b)->symType = type; (&p.b)->storage = StorageNone; (&p.b)->npointers = 0;};
        { (&p)->name = id->name; (&p)->linkName = id->name; (&p)->pos = id->p; (&p)->b = p.b; (&p)->u.s = NULL; (&p)->next = NULL;};
        p.u.s = NULL;
        if (! symTabIsMember(s_symTab,&p,&ii,&pp)){
                {pp = (S_symbol*) stackMemoryAlloc(sizeof(S_symbol)); };
                *pp = p;
                {pp->u.s = (S_symStructSpecific*) stackMemoryAlloc(sizeof(S_symStructSpecific)); };
                { (pp->u.s)->super = NULL; (pp->u.s)->records = NULL; (pp->u.s)->casts.node = NULL; (pp->u.s)->casts.sub = NULL; (pp->u.s)->nnested = 0; (pp->u.s)->nest = NULL; (pp->u.s)->stype.m = type; { (&((&(pp->u.s)->stype.u)->f))->args = NULL; (&((&(pp->u.s)->stype.u)->f))->thisFunList = NULL;}; (pp->u.s)->stype.typedefin = NULL; (pp->u.s)->stype.next = NULL; (pp->u.s)->sptrtype.m = TypePointer; { (&((&(pp->u.s)->sptrtype.u)->f))->args = NULL; (&((&(pp->u.s)->sptrtype.u)->f))->thisFunList = NULL;}; (pp->u.s)->sptrtype.typedefin = NULL; (pp->u.s)->sptrtype.next = &pp->u.s->stype; (pp->u.s)->currPackage = 0; (pp->u.s)->existsDEIarg = 0; (pp->u.s)->classFile = -1; (pp->u.s)->recSearchCounter = 0;};
                pp->u.s->stype.u.t = pp;
                setGlobalFileDepNames(id->name, pp, MEM_XX);
                addSymbol(pp, s_symTab);
        }
        addCxReference(pp, &id->p, usage,s_noneFileIndex, s_noneFileIndex);
        return(&pp->u.s->stype);
}
static int isStaticFun(S_symbol *p) {
        return(p->b.symType==TypeDefault && p->b.storage==StorageStatic
                   && p->u.type->m == TypeFunction);
}
void setGlobalFileDepNames(char *iname, S_symbol *pp, int memory) {
        char *mname, *fname;
        char tmp[MACRO_NAME_SIZE];
        S_symbol *memb;
        int ii,rr,filen, order, len, len2;
        if (iname == NULL) iname="";
        { if (!(pp)) internalCheckFail("pp", "semact.c", 787);};
        if (s_opt.exactPositionResolve) {
                fname = simpleFileName(s_fileTab.tab[pp->pos.file]->name);
                sprintf(tmp, "%x-%s-%x-%x%c",
                                hashFun(s_fileTab.tab[pp->pos.file]->name),
                                fname, pp->pos.line, pp->pos.coll,
                                '!');
        } else if (iname[0]==0) {
                filen = pp->pos.file;
                pp->name=iname; pp->linkName=iname;
                order = 0;
                rr = symTabIsMember(s_symTab, pp, &ii, &memb);
                while (rr) {
                        if (memb->pos.file==filen) order++;
                        rr = symTabNextMember(pp, &memb);
                }
                fname = simpleFileName(s_fileTab.tab[filen]->name);
                sprintf(tmp, "%s%c%d%c", fname, SLASH, order, '!');
        } else {
                tmp[0] = 0;
        }
        len = strlen(tmp);
        len2 = len + strlen(iname);
        { if (!(len < MACRO_NAME_SIZE-2)) internalCheckFail("len < MACRO_NAME_SIZE-2", "semact.c", 815);};
        if (memory == MEM_XX) {
                {mname = (char*) stackMemoryAlloc((len2+1)*sizeof(char)); };
        } else {
                {{ { if (!((len2+1) >= 0)) internalCheckFail("(len2+1) >= 0", "semact.c", 819);}; ppmMemoryi = ((char*)(((char*)(ppmMemory+ppmMemoryi))+(STANDARD_ALLIGNEMENT-1-((((unsigned)(ppmMemory+ppmMemoryi))-1) & (STANDARD_ALLIGNEMENT-1))))) - ppmMemory; if (ppmMemoryi+(len2+1)*sizeof(char) >= SIZE_ppmMemory) { fatalError(ERR_NO_MEMORY,"ppmMemory", 65); } mname = (char*) (ppmMemory + ppmMemoryi); ppmMemoryi += (len2+1)*sizeof(char);};};
        }
        strcpy(mname, tmp);
        strcpy(mname+len,iname);
        pp->name = mname + len;
        pp->linkName = mname;
}
S_typeModifiers *crNewAnnonymeStrUnion(S_idIdent *typeName) {
        S_symbol *pp;
        int type,ti,line,coll,nlen,nnlen;
        char *name,*fname,*fsname;
        char ttt[TMP_STRING_SIZE];
        { if (!(typeName)) internalCheckFail("typeName", "semact.c", 858);};
        { if (!(typeName->sd)) internalCheckFail("typeName->sd", "semact.c", 859);};
        { if (!(typeName->sd->b.symType == TypeKeyword)) internalCheckFail("typeName->sd->b.symType == TypeKeyword", "semact.c", 860);};
        { if (!(typeName->sd->u.keyWordVal == 319 || typeName->sd->u.keyWordVal == 328 || typeName->sd->u.keyWordVal == 320)) internalCheckFail("typeName->sd->u.keyWordVal == 319 || typeName->sd->u.keyWordVal == 328 || typeName->sd->u.keyWordVal == 320", "semact.c", 861);};
        if (typeName->sd->u.keyWordVal == 319) type = TypeStruct;
        else type = TypeUnion;
        pp = ((S_symbol*) stackMemoryAlloc(sizeof(S_symbol)));
        { (&pp->b)->record = 0; (&pp->b)->accessFlags = 0; (&pp->b)->javaSourceLoaded = 0; (&pp->b)->javaFileLoaded = 0; (&pp->b)->symType = type; (&pp->b)->storage = StorageNone; (&pp->b)->npointers = 0;};
        { (pp)->name = ""; (pp)->linkName = NULL; (pp)->pos = typeName->p; (pp)->b = pp->b; (pp)->u.type = NULL; (pp)->next = NULL;};
        setGlobalFileDepNames("", pp, MEM_XX);
        {pp->u.s = (S_symStructSpecific*) stackMemoryAlloc(sizeof(S_symStructSpecific)); };
        { (pp->u.s)->super = NULL; (pp->u.s)->records = NULL; (pp->u.s)->casts.node = NULL; (pp->u.s)->casts.sub = NULL; (pp->u.s)->nnested = 0; (pp->u.s)->nest = NULL; (pp->u.s)->stype.m = type; { (&((&(pp->u.s)->stype.u)->f))->args = NULL; (&((&(pp->u.s)->stype.u)->f))->thisFunList = NULL;}; (pp->u.s)->stype.typedefin = NULL; (pp->u.s)->stype.next = NULL; (pp->u.s)->sptrtype.m = TypePointer; { (&((&(pp->u.s)->sptrtype.u)->f))->args = NULL; (&((&(pp->u.s)->sptrtype.u)->f))->thisFunList = NULL;}; (pp->u.s)->sptrtype.typedefin = NULL; (pp->u.s)->sptrtype.next = &pp->u.s->stype; (pp->u.s)->currPackage = 0; (pp->u.s)->existsDEIarg = 0; (pp->u.s)->classFile = -1; (pp->u.s)->recSearchCounter = 0;};
        pp->u.s->stype.u.t = pp;
        addSymbol(pp, s_symTab);
        return(&pp->u.s->stype);
}
void specializeStrUnionDef(S_symbol *sd, S_symbol *rec) {
        S_symbol *dd;
        { if (!(sd->b.symType == TypeStruct || sd->b.symType == TypeUnion)) internalCheckFail("sd->b.symType == TypeStruct || sd->b.symType == TypeUnion", "semact.c", 895);};
        { if (!(sd->u.s)) internalCheckFail("sd->u.s", "semact.c", 896);};
        if (sd->u.s->records!=NULL) return;
        sd->u.s->records = rec;
        addToTrail(setToNull, & (sd->u.s->records) );
        for(dd=rec; dd!=NULL; dd=dd->next) {
                dd->linkName = string3ConcatInStackMem(sd->linkName,".",dd->name);
                dd->b.record = 1;
                if ( ((s_language & (16)) != 0) && dd->b.symType==TypeDefault
                                && dd->u.type->m==TypeFunction) {
                        dd->u.type->u.f.thisFunList = &sd->u.s->records;
                }
                addCxReference(dd,&dd->pos,UsageDefined,s_noneFileIndex, s_noneFileIndex);
        }
}
S_typeModifiers *simpleEnumSpecifier(S_idIdent *id, int usage) {
        S_symbol p,*pp;
        int ii;
        { (&p.b)->record = 0; (&p.b)->accessFlags = 0; (&p.b)->javaSourceLoaded = 0; (&p.b)->javaFileLoaded = 0; (&p.b)->symType = TypeEnum; (&p.b)->storage = StorageNone; (&p.b)->npointers = 0;};
        { (&p)->name = id->name; (&p)->linkName = id->name; (&p)->pos = id->p; (&p)->b = p.b; (&p)->u.enums = NULL; (&p)->next = NULL;};
        p.u.enums = NULL;
        if (! symTabIsMember(s_symTab,&p,&ii,&pp)) {
                pp = ((S_symbol*) stackMemoryAlloc(sizeof(S_symbol)));
                *pp = p;
                addSymbol(pp, s_symTab);
        }
        addCxReference(pp, &id->p, usage,s_noneFileIndex, s_noneFileIndex);
        return(crSimpleEnumType(pp,TypeEnum));
}
S_typeModifiers *crNewAnnonymeEnum(S_symbolList *enums) {
        S_symbol *pp;
        pp = ((S_symbol*) stackMemoryAlloc(sizeof(S_symbol)));
        { (&pp->b)->record = 0; (&pp->b)->accessFlags = 0; (&pp->b)->javaSourceLoaded = 0; (&pp->b)->javaFileLoaded = 0; (&pp->b)->symType = TypeEnum; (&pp->b)->storage = StorageNone; (&pp->b)->npointers = 0;};
        { (pp)->name = ""; (pp)->linkName = ""; (pp)->pos = s_noPos; (pp)->b = pp->b; (pp)->u.enums = enums; (pp)->next = NULL;};
        pp->u.enums = enums;
        return(crSimpleEnumType(pp,TypeEnum));
}
